@@ -205,7 +205,8 @@ load_vmis(const char *file)
 	  exit(1);
 	} else
 	{ e4--;				/* backspace over ) */
-	  e5 -= is_vmh;
+	  if ( e5 )
+	    e5 -= is_vmh;
 	}
 
 	vmi_list[vmi_count].name  = my_strndup(s1, e1-s1);
@@ -282,7 +283,7 @@ mystrlwr(char *to, const char *from)	/* my*: Windows name conflict */
 { char *s;
 
   for(s=to; *from; from++)
-  { *s++ = tolower(*from);
+  { *s++ = (char) tolower(*from);
   }
   *s = '\0';
 
