@@ -289,7 +289,7 @@ CpuTime(cputime_kind which)
 #define CPU_TIME_DONE
   struct tms t;
   double used;
-  static int MTOK_got_hz = FALSE;
+  static int MTOK_got_hz = false;
   static double MTOK_hz;
 
   if ( !MTOK_got_hz )
@@ -379,12 +379,16 @@ WallTime(void)
 
 int
 CpuCount(void)
-{ return sysconf(_SC_NPROCESSORS_CONF);
+{
+#ifdef _SC_NPROCESSORS_ONLN
+  return (int)sysconf(_SC_NPROCESSORS_ONLN);
+#else
+  return (int)sysconf(_SC_NPROCESSORS_CONF);
+#endif
 }
 
-#else
+#elif defined(PROCFS_CPUINFO)
 
-#ifdef PROCFS_CPUINFO
 int
 CpuCount(void)
 { FILE *fd = fopen("/proc/cpuinfo", "r");
@@ -419,9 +423,7 @@ CpuCount(void)
   return 0;
 }
 
-#else /*PROCFS_CPUINFO*/
-
-#ifdef HAVE_SYSCTLBYNAME	/* MacOS X */
+#elif defined(HAVE_SYSCTLBYNAME)	/* MacOS X */
 
 #include <sys/param.h>
 #include <sys/sysctl.h>
@@ -444,19 +446,14 @@ CpuCount(void)
 { return 0;
 }
 
-#endif /*sysctlbyname*/
-
-#endif /*PROCFS_CPUINFO*/
-
-#endif /*HAVE_SC_NPROCESSORS_CONF*/
-
+#endif /*Get CpuCount*/
 
 void
 setOSPrologFlags(void)
 { int cpu_count = CpuCount();
 
   if ( cpu_count > 0 )
-    PL_set_prolog_flag("cpu_count", PL_INTEGER, (intptr_t)cpu_count);
+    PL_set_prolog_flag("cpu_count", PL_INTEGER, (intptr_t)cpu_count); /* dubious cast */
 }
 #endif
 
@@ -559,7 +556,7 @@ _PL_Random(void)
 
   if ( !LD->os.rand_initialised )
   { setRandom(NULL);
-    LD->os.rand_initialised = TRUE;
+    LD->os.rand_initialised = true;
   }
 
 #ifdef HAVE_RANDOM
@@ -703,7 +700,7 @@ verify_tmp_dir(const char* tmpdir)
 { const char *reason = NULL;
 
   if ( tmpdir == NULL )
-    return FALSE;
+    return false;
 
   if ( !ExistsDirectory(tmpdir) )
     reason = "no such directory";
@@ -714,10 +711,10 @@ verify_tmp_dir(const char* tmpdir)
 		      PL_CHARS, tmpdir,
 		      PL_CHARS, reason) )
     { /* to prevent ignoring return value warning */ }
-    return FALSE;
+    return false;
   }
 
-  return TRUE;
+  return true;
 }
 
 #ifdef O_XOS
@@ -813,7 +810,7 @@ retry:
     PL_UNLOCK(L_OS);
   }
 
-  addNewHTable(GD->os.tmp_files, (table_key_t)tname, TRUE);
+  addNewHTable(GD->os.tmp_files, (table_key_t)tname, true);
 
   return tname;
 }
@@ -822,7 +819,7 @@ retry:
 int
 DeleteTemporaryFile(atom_t name)
 { GET_LD
-  int rc = FALSE;
+  int rc = false;
 
   if ( GD->os.tmp_files )
   { if ( GD->os.tmp_files )
@@ -1012,7 +1009,7 @@ initExpand(void)
     { char *e;
 
       if ( (e = strchr(cpaths, ':')) )
-      { int l = e-cpaths;
+      { size_t l = e-cpaths;
 
 	strncpy(buf, cpaths, l);
 	buf[l] = EOS;
@@ -1158,7 +1155,7 @@ cleanupExpand(void)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 verify_entry() verifies the path cache for this   path is still safe. If
-not it updates the cache and returns FALSE.
+not it updates the cache and returns false.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
@@ -1169,20 +1166,20 @@ verify_entry(CanonicalDir d)
   if ( statfunc(OsPath(d->canonical, tmp), &buf) == 0 )
   { if ( d->inode  == buf.st_ino &&
 	 d->device == buf.st_dev )
-      return TRUE;
+      return true;
 
     DEBUG(MSG_OS_DIR, Sdprintf("%s: inode/device changed\n", d->canonical));
 
     d->inode  = buf.st_ino;
     d->device = buf.st_dev;
-    return TRUE;
+    return true;
   } else
   { DEBUG(MSG_OS_DIR, Sdprintf("%s: no longer exists\n", d->canonical));
 
     deleteCanonicalDir(d);
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1989,12 +1986,12 @@ ChDir(const char *path)
   OsPath(path, ospath);
 
   if ( path[0] == EOS || streq(path, ".") || is_cwd(path) )
-    return TRUE;
+    return true;
 
   if ( !AbsoluteFile(path, tmp, sizeof(tmp)) )
-    return FALSE;
+    return false;
   if ( is_cwd(tmp) )
-    return TRUE;
+    return true;
 
   if ( chdir(ospath) == 0 )
   { size_t len;
@@ -2011,10 +2008,10 @@ ChDir(const char *path)
     GD->paths.CWDdir = store_string(tmp);
     PL_UNLOCK(L_OS);
 
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -2147,7 +2144,7 @@ PopTty(IOSTREAM *s, ttybuf *buf)
 
 int
 Sttymode(IOSTREAM *s)
-{ return true(s, SIO_RAW) ? TTY_RAW : TTY_COOKED;
+{ return ison(s, SIO_RAW) ? TTY_RAW : TTY_COOKED;
 }
 
 static void
@@ -2169,31 +2166,34 @@ ResetStdin(void)
 static ssize_t
 Sread_terminal(void *handle, char *buf, size_t size)
 { GET_LD
-  intptr_t h = (intptr_t)handle;
-  int fd = (int)h;
+  ssize_t rc;
   source_location oldsrc = LD->read_source;
 
-  if ( LD->prompt.next &&
-       false(Sinput, SIO_RAW) &&
-       true(Sinput, SIO_ISATTY) )
-    PL_write_prompt(TRUE);
-  else if ( true(Soutput, SIO_ISATTY) )
-    Sflush(Suser_output);
+  if ( Sinput->handle == handle )
+  { if ( LD->prompt.next &&
+	 isoff(Sinput, SIO_RAW) &&
+	 ison(Sinput, SIO_ISATTY) )
+      PL_write_prompt(true);
+    else if ( ison(Soutput, SIO_ISATTY) )
+      Sflush(Suser_output);
 
-  PL_dispatch(fd, PL_DISPATCH_WAIT);
-  size = (*GD->os.org_terminal.read)(handle, buf, size);
+    PL_dispatch(Sinput, PL_DISPATCH_WAIT);
+    rc = (*GD->os.org_terminal.read)(handle, buf, size);
 
-  if ( size == 0 )			/* end-of-file */
-  { if ( fd == 0 )
-    { Sclearerr(Suser_input);
-      LD->prompt.next = TRUE;
-    }
-  } else if ( size > 0 && buf[size-1] == '\n' )
-    LD->prompt.next = TRUE;
+    if ( rc == 0 )			/* end-of-file */
+    { if ( Sinput == Suser_input )
+      { Sclearerr(Suser_input);
+	LD->prompt.next = true;
+      }
+    } else if ( rc > 0 && buf[rc-1] == '\n' )
+      LD->prompt.next = true;
 
-  LD->read_source = oldsrc;
+    LD->read_source = oldsrc;
+  } else
+  { rc = (*GD->os.org_terminal.read)(handle, buf, size);
+  }
 
-  return size;
+  return rc;
 }
 
 void
@@ -2209,7 +2209,7 @@ ResetTty(void)
     Soutput->functions =
     Serror->functions  = &GD->os.iofunctions;
   }
-  LD->prompt.next = TRUE;
+  LD->prompt.next = true;
 }
 
 #ifdef O_HAVE_TERMIO			/* sys/termios.h or sys/termio.h */
@@ -2229,24 +2229,26 @@ GetTtyState(int fd, struct termios *tio)
 
 #ifdef HAVE_TCSETATTR
   if ( tcgetattr(fd, tio) )
-    return FALSE;
+    return false;
 #else
   if ( ioctl(fd, TIOCGETA, tio) )
-    return FALSE;
+    return false;
 #endif
 
-  return TRUE;
+  return true;
 }
 
-static int
+static bool
 SetTtyState(int fd, struct termios *tio)
 {
 #ifdef HAVE_TCSETATTR
   if ( tcsetattr(fd, TCSANOW, tio) != 0 )
-  { static int MTOK_warned;			/* MT-OK */
+  { static bool MTOK_warned;			/* MT-OK */
 
-    if ( !MTOK_warned++ )
+    if ( !MTOK_warned )
+    { MTOK_warned = true;
       return warning("Failed to set terminal: %s", OsError());
+    }
   }
 #else
 #ifdef TIOCSETAW
@@ -2258,9 +2260,13 @@ SetTtyState(int fd, struct termios *tio)
 #endif
 
   if ( fd == ttyfileno && ttytab.state )
-    ttymodified = memcmp(&TTY_STATE(&ttytab), tio, sizeof(*tio));
+    ttymodified = memcmp(&TTY_STATE(&ttytab), tio, sizeof(*tio)) != 0;
 
-  return TRUE;
+  DEBUG(MSG_TTY,
+	Sdprintf("[%d] SetTtyState(%d): modified = %d\n",
+		 PL_thread_self(), fd, ttymodified));
+
+  return true;
 }
 
 
@@ -2273,7 +2279,7 @@ PushTty(IOSTREAM *s, ttybuf *buf, int mode)
   buf->mode  = Sttymode(s);
   buf->state = NULL;
 
-  if ( false(s, SIO_ISATTY) )
+  if ( isoff(s, SIO_ISATTY) )
   { DEBUG(MSG_TTY, Sdprintf("stdin is not a terminal\n"));
     succeed;				/* not a terminal */
   }
@@ -2290,7 +2296,7 @@ PushTty(IOSTREAM *s, ttybuf *buf, int mode)
   buf->state = allocHeapOrHalt(sizeof(tty_state));
 
   if ( !GetTtyState(fd, &TTY_STATE(buf)) )
-    return FALSE;
+    return false;
 
   tio = TTY_STATE(buf);			/* structure copy */
 
@@ -2320,13 +2326,13 @@ PushTty(IOSTREAM *s, ttybuf *buf, int mode)
 
 /**
  * @param do_free is one of
- *   - FALSE: do not free the state
- *   - TRUE:  free the state
+ *   - false: do not free the state
+ *   - true:  free the state
  */
 
 bool
 PopTty(IOSTREAM *s, ttybuf *buf, int do_free)
-{ int rc = TRUE;
+{ int rc = true;
 
   Sset_ttymode(s, buf->mode);
 
@@ -2432,7 +2438,7 @@ PopTty(IOSTREAM *s, ttybuf *buf, int do_free)
 
   Sset_ttymode(s, buf->mode);
   if ( buf->mode != TTY_RAW )
-    LD->prompt.next = TRUE;
+    LD->prompt.next = true;
 
   succeed;
 }
@@ -2505,7 +2511,7 @@ int
 Setenv(char *name, char *value)
 {
 #ifdef HAVE_SETENV
-  if ( setenv(name, value, TRUE) != 0 )
+  if ( setenv(name, value, true) != 0 )
     return PL_error(NULL, 0, MSG_ERRNO, ERR_SYSCALL, "setenv");
 #else
   char *buf;
@@ -2939,7 +2945,8 @@ findExecutable(const char *av0, char *buffer, size_t buflen)
 
 #if __unix__				/* argv[0] can be an #! script! */
   if ( file )
-  { int n, fd;
+  { size_t n;
+    int fd;
     char buf[PATH_MAX];
 					/* Fails if mode is x-only, but */
 					/* then it can't be a script! */
@@ -3094,7 +3101,7 @@ accurate, interruptable and restartable.
 #if !defined(PAUSE_DONE) && defined(HAVE_NANOSLEEP)
 #define PAUSE_DONE 1
 
-int
+bool
 Pause(double t)
 { struct timespec req;
   int rc;
@@ -3109,9 +3116,9 @@ Pause(double t)
   { rc = nanosleep(&req, &req);
     if ( rc == -1 && errno == EINTR )
     { if ( PL_handle_signals() < 0 )
-	return FALSE;
+	return false;
     } else
-      return TRUE;
+      return true;
   }
 }
 
@@ -3121,14 +3128,14 @@ Pause(double t)
 #if !defined(PAUSE_DONE) && defined(HAVE_USLEEP)
 #define PAUSE_DONE 1
 
-int
+bool
 Pause(double t)
 { if ( t <= 0.0 )
-    return TRUE;
+    return true;
 
   usleep((unsigned long)(t * 1000000.0));
 
-  return TRUE;
+  return true;
 }
 
 #endif /*HAVE_USLEEP*/
@@ -3137,7 +3144,7 @@ Pause(double t)
 #if !defined(PAUSE_DONE) && defined(HAVE_SELECT)
 #define PAUSE_DONE 1
 
-int
+bool
 Pause(double time)
 { struct timeval timeout;
 
@@ -3149,7 +3156,7 @@ Pause(double time)
     timeout.tv_usec = (long)(time * 1000000) % 1000000;
     select(1, NULL, NULL, NULL, &timeout);
 
-    return TRUE;
+    return true;
   } else
   { int rc;
     int left = (int)(time+0.5);
@@ -3158,9 +3165,9 @@ Pause(double time)
     { rc = sleep(left);
       if ( rc == -1 && errno == EINTR )
       { if ( PL_handle_signals() < 0 )
-	  return FALSE;
+	  return false;
 
-	return TRUE;
+	return true;
       }
       left -= rc;
     } while ( rc != 0 );
@@ -3172,14 +3179,14 @@ Pause(double time)
 #if !defined(PAUSE_DONE) && defined(HAVE_DOSSLEEP)
 #define PAUSE_DONE 1
 
-int					/* a millisecond granualrity. */
+bool					/* a millisecond granualrity. */
 Pause(double time)			/* the EMX function sleep uses seconds */
 { if ( time <= 0.0 )			/* the select() trick does not work at all. */
-    return TRUE;
+    return true;
 
   DosSleep((ULONG)(time * 1000));
 
-  return TRUE;
+  return true;
 }
 
 #endif /*HAVE_DOSSLEEP*/
@@ -3187,7 +3194,7 @@ Pause(double time)			/* the EMX function sleep uses seconds */
 #if !defined(PAUSE_DONE) && defined(HAVE_SLEEP)
 #define PAUSE_DONE 1
 
-int
+bool
 Pause(double t)
 { if ( t <= 0.5 )
     succeed;
@@ -3202,17 +3209,17 @@ Pause(double t)
 #if !defined(PAUSE_DONE) && defined(HAVE_DELAY)
 #define PAUSE_DONE 1
 
-int
+bool
 Pause(double t)
 { delay((int)(t * 1000));
 
-  return TRUE;
+  return true;
 }
 
 #endif /*HAVE_DELAY*/
 
 #ifndef PAUSE_DONE
-int
+bool
 Pause(double t)
 { return notImplemented("sleep", 1);
 }
