@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2023, University of Amsterdam
+    Copyright (c)  2011-2025, University of Amsterdam
 			      VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -51,7 +51,7 @@
 #undef LD
 #define LD LOCAL_LD
 
-static int text_representation_error(PL_chars_t *text, IOENC enc);
+static bool text_representation_error(PL_chars_t *text, IOENC enc);
 static int text_error(PL_chars_t *text, int rc);
 
 		 /*******************************
@@ -148,32 +148,26 @@ PL_from_stack_text(PL_chars_t *text, int flags)
 }
 
 
-size_t
-PL_text_length(const PL_chars_t *text)
-{ assert(text->canonical);
-
 #if SIZEOF_WCHAR_T == 2
-  if ( text->encoding == ENC_WCHAR )
-  { const wchar_t *s = (const wchar_t *)text->text.t;
-    const wchar_t *e = &s[text->length];
-    size_t count = 0;
+size_t
+utf16_text_length(const PL_chars_t *text)
+{ assert(text->encoding == ENC_WCHAR);
 
-    while(s < e)
-    { int c = *s++;
+  const wchar_t *s = (const wchar_t *)text->text.t;
+  const wchar_t *e = &s[text->length];
+  size_t count = 0;
 
-      count++;
-      if ( IS_UTF16_LEAD(c) )
-	s++;
-    }
+  while(s < e)
+  { int c = *s++;
 
-    return count;
+    count++;
+    if ( IS_UTF16_LEAD(c) )
+      s++;
   }
-#endif
 
-  return text->length;
+  return count;
 }
-
-
+#endif
 
 #define INT64_DIGITS 20
 
@@ -530,14 +524,14 @@ globalSpaceRequirement(PL_chars_t *text)
 #define unify_text(term, tail, text, type) \
 	LDFUNC(unify_text, term, tail, text, type)
 
-static int
+static bool
 unify_text(DECL_LD term_t term, term_t tail, PL_chars_t *text, int type)
 { switch(type)
   { case PL_ATOM:
     { atom_t a = textToAtom(text);
 
       if ( a )
-      { int rval = PL_unify_atomic(term, a);
+      { bool rval = PL_unify_atomic(term, a);
 
 	PL_unregister_atom(a);
 	return rval;
@@ -602,7 +596,7 @@ unify_text(DECL_LD term_t term, term_t tail, PL_chars_t *text, int type)
 	  { const pl_wchar_t *s = (const pl_wchar_t *)text->text.t;
 	    const pl_wchar_t *e = &s[text->length];
 
-	    if ( !(p0 = p = INIT_SEQ_STRING(text->length)) )
+	    if ( !(p0 = p = INIT_SEQ_STRING(PL_text_length(text))) )
 	      return false;
 
 	    if ( type == PL_CODE_LIST )
@@ -691,7 +685,7 @@ unify_text(DECL_LD term_t term, term_t tail, PL_chars_t *text, int type)
 	  }
 	}
 
-	int rc = CLOSE_SEQ_STRING(p, p0, tail, term, l );
+	bool rc = CLOSE_SEQ_STRING(p, p0, tail, term, l );
 	PL_reset_term_refs(l);
 	return rc;
       }
@@ -705,10 +699,10 @@ unify_text(DECL_LD term_t term, term_t tail, PL_chars_t *text, int type)
 }
 
 
-int
+bool
 PL_unify_text(term_t term, term_t tail, PL_chars_t *text, int type)
 { GET_LD
-  int rc;
+  bool rc;
 
   PL_STRINGS_MARK();
   rc = unify_text(term, tail, text, type);
@@ -898,7 +892,7 @@ PL_promote_text(PL_chars_t *text)
 }
 
 
-static int
+static bool
 PL_demote_text(PL_chars_t *text, int flags)
 { if ( text->encoding != ENC_ISO_LATIN_1 )
   { if ( text->storage == PL_CHARS_MALLOC )
@@ -957,7 +951,8 @@ PL_demote_text(PL_chars_t *text, int flags)
     }
   }
 
-  succeed;
+  text->canonical = true;
+  return true;
 }
 
 
@@ -1105,7 +1100,8 @@ rep_error:
   if ( (flags & CVT_EXCEPTION) )
   { char msg[128];
 
-    sprintf(msg,
+    snprintf(msg,
+	    sizeof msg,
 	    "Cannot represent char U%04x using %s encoding",
 	    norep,
 	    target == ENC_ISO_LATIN_1 ? "ISO Latin-1" : "current locale");
@@ -1159,7 +1155,7 @@ PL_canonicalise_text(PL_chars_t *text)
       case ENC_WCHAR:
       { const pl_wchar_t *w;
 	const pl_wchar_t *e;
-	int wide;
+	bool wide;
 
 #if SIZEOF_WCHAR_T == 2
       case_wchar:
@@ -1190,6 +1186,7 @@ PL_canonicalise_text(PL_chars_t *text)
 	if ( !wide )
 	  return PL_demote_text(text, 0);
 
+	text->canonical = true;
 	return true;
       }
       case ENC_UTF16LE:		/* assume text->length is in bytes */
@@ -1440,7 +1437,7 @@ PL_canonicalise_text(PL_chars_t *text)
 }
 
 
-static int
+static bool
 text_representation_error(PL_chars_t *text, IOENC enc)
 { char msg[100];
 
