@@ -223,10 +223,15 @@ freeHeap(void *mem, size_t n)
 
 void
 linger_always(linger_list** list, void (*unalloc)(void *), void *object)
-{ if ( GD->cleaning != CLN_DATA )
+{ if ( GD->halt.cleaning != CLN_DATA )
   { linger_list *c = allocHeapOrHalt(sizeof(*c));
     linger_list *o;
 
+    DEBUG(0,
+	  for(linger_list *l = *list; l; l = l->next)
+	  { assert(l->object != object);
+	  });
+
     c->generation = global_generation();
     c->object	  = object;
     c->unalloc	  = unalloc;
@@ -263,8 +268,8 @@ free_lingering(linger_list **list, gen_t generation)
 		*             STACKS            *
 		*********************************/
 
-int
-enableSpareStack(Stack s, int always)
+bool
+enableSpareStack(Stack s, bool always)
 { if ( s->spare && (roomStackP(s) < s->def_spare || always) )
   { DEBUG(MSG_SPARE_STACK,
 	  Sdprintf("Enabling spare on %s: %zd bytes\n", s->name, s->spare));
@@ -715,11 +720,15 @@ outOfStack(void *stack, stack_overflow_action how)
       }
       exception_term = exception_bin;
 
+#if O_THROW
       if ( how == STACK_OVERFLOW_THROW &&
 	   LD->exception.throw_environment )
       {						/* see PL_throw() */
 	longjmp(LD->exception.throw_environment->exception_jmp_env, 1);
       }
+#else
+      PL_fatal_error("Could not handle stack overflow");
+#endif
 
       return false;
     }
@@ -731,7 +740,7 @@ outOfStack(void *stack, stack_overflow_action how)
 
 
 bool
-raiseStackOverflow(int overflow)
+raiseStackOverflow(boolex_t overflow)
 { GET_LD
   Stack s;
 
@@ -799,14 +808,8 @@ Word
 allocGlobal(DECL_LD size_t n)
 { Word result;
 
-  if ( !hasGlobalSpace(n) )
-  { int rc;
-
-    if ( (rc=ensureGlobalSpace(n, ALLOW_GC)) != true )
-    { raiseStackOverflow(rc);
-      return NULL;
-    }
-  }
+  if ( !ensureGlobalSpace(n, ALLOW_GC) )
+    return NULL;
 
   result = gTop;
   gTop += n;
@@ -1002,17 +1005,15 @@ packed into two `guards words'. We  cannot   just  copy the double as it
 might not be properly aligned.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+bool
 put_double(DECL_LD Word at, double d, int flags)
 { Word p;
   word m = mkIndHdr(WORDS_PER_DOUBLE, TAG_FLOAT);
 
-  if ( flags != ALLOW_CHECKED && !hasGlobalSpace(2+WORDS_PER_DOUBLE) )
-  { int rc = ensureGlobalSpace(2+WORDS_PER_DOUBLE, flags);
+  if ( flags != ALLOW_CHECKED &&
+       !ensureGlobalSpace(2+WORDS_PER_DOUBLE, flags) )
+    return false;
 
-    if ( rc != true )
-      return rc;
-  }
   p = gTop;
   gTop += 2+WORDS_PER_DOUBLE;
 
@@ -1034,7 +1035,7 @@ put_double(DECL_LD Word at, double d, int flags)
 		 *  GENERIC INDIRECT OPERATIONS	*
 		 *******************************/
 
-int
+bool
 equalIndirect(word w1, word w2)
 { Word p1 = addressIndirect(w1);
   Word p2 = addressIndirect(w2);
@@ -1042,15 +1043,10 @@ equalIndirect(word w1, word w2)
   if ( *p1 == *p2 )
   { size_t n = wsizeofInd(*p1);
 
-    while( n-- > 0 )
-    { if ( *++p1 != *++p2 )
-	fail;
-    }
-
-    succeed;
+    return memcmp(p1+1, p2+1, n*sizeof(word)) == 0;
   }
 
-  fail;
+  return false;
 }
 
 word
@@ -1831,7 +1827,7 @@ initPTMalloc(void)
 }
 
 
-int
+bool
 initMalloc(void)
 { return ( initTCMalloc() ||
 	   initPTMalloc() ||
