@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2022, VU University Amsterdam
+    Copyright (c)  2013-2025, VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
@@ -43,6 +43,7 @@
 #include "pl-fli.h"
 #include "pl-gc.h"
 #include "pl-gvar.h"
+#include "pl-copyterm.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Dicts are associative arrays,  where  keys   are  either  atoms  or small
@@ -66,7 +67,9 @@ The term has the following layout on the global stack:
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags);
+static bool PL_get_dict_ex(term_t data,
+			   term_t tag, term_t dict, int flags);
+
 #define DICT_GET_ALL	0xff
 #define DICT_GET_PAIRS	0x01
 #define DICT_GET_EQUALS	0x02
@@ -74,7 +77,7 @@ static int PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags);
 #define DICT_GET_TERM	0x08
 
 functor_t
-dict_functor(int pairs)
+dict_functor(size_t pairs)
 { if ( pairs < CACHED_DICT_FUNCTORS )
   { if ( GD->dict.dict_functors[pairs] )
       return GD->dict.dict_functors[pairs];
@@ -86,22 +89,42 @@ dict_functor(int pairs)
   }
 }
 
+static bool
+is_dict_functor(functor_t f)
+{ FunctorDef fd = valueFunctor(f);
+
+  return ( fd->name == ATOM_dict &&
+	   fd->arity%2 == 1 );
+}
+
+#define is_dict_term(w) \
+	LDFUNC(is_dict_term, w)
+
+static bool
+is_dict_term(DECL_LD word w)
+{ if ( isTerm(w) )
+  { Functor f = valueTerm(w);
+
+    return is_dict_functor(f->definition);
+  }
+
+  return false;
+}
+
 		 /*******************************
 		 *      LOW-LEVEL FUNCTIONS	*
 		 *******************************/
 
 #define get_dict_ex(t, dp, ex) LDFUNC(get_dict_ex, t, dp, ex)
-static int
-get_dict_ex(DECL_LD term_t t, Word dp, int ex)
+static bool
+get_dict_ex(DECL_LD term_t t, Word dp, bool ex)
 { Word p = valTermRef(t);
 
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 )		/* does *not* validate ordering */
+    if ( is_dict_functor(f->definition) )
     { *dp = *p;
       return true;
     }
@@ -110,8 +133,7 @@ get_dict_ex(DECL_LD term_t t, Word dp, int ex)
   if ( !ex )
     return false;
 
-  PL_type_error("dict", t);
-  return false;
+  return PL_type_error("dict", t),false;
 }
 
 
@@ -122,17 +144,15 @@ and is never a reference.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define get_create_dict_ex(t, dt) LDFUNC(get_create_dict_ex, t, dt)
-static int
+static bool
 get_create_dict_ex(DECL_LD term_t t, term_t dt)
 { Word p = valTermRef(t);
 
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 )		/* does *not* validate ordering */
+    if ( is_dict_functor(f->definition) )
     { *valTermRef(dt) = *p;
       return true;
     }
@@ -152,23 +172,26 @@ dict_lookup_ptr() returns a pointer to the value for a given key
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 Word
-dict_lookup_ptr(DECL_LD word dict, word name)
+dict_lookup_ptr(DECL_LD word dict, word name, size_t *arg)
 { Functor data = valueTerm(dict);
-  int arity = arityFunctor(data->definition);
-  int l = 1, h = arity/2;
+  size_t arity = arityFunctor(data->definition);
+  size_t l = 1, h = arity/2;
 
   if ( arity == 1 )
     return NULL;			/* empty */
   assert(arity%2 == 1);
 
   for(;;)
-  { int m = (l+h)/2;
+  { size_t m = (l+h)/2;
     Word p;
 
     deRef2(&data->arguments[m*2], p);
 
     if ( *p == name )
+    { if ( arg )
+	*arg = m;
       return p-1;
+    }
 
     if ( l == h )
       return NULL;
@@ -192,9 +215,9 @@ dict_lookup_ptr(DECL_LD word dict, word name)
 */
 
 #define dict_ordered(data, count, dupl) LDFUNC(dict_ordered, data, count, dupl)
-static int
-dict_ordered(DECL_LD Word data, int count, Word dupl)
-{ int ordered = true;
+static bool
+dict_ordered(DECL_LD Word data, size_t count, Word dupl)
+{ bool ordered = true;
   Word n1, n2;
 
   if ( count > 0 )
@@ -228,7 +251,7 @@ dict_ordered(DECL_LD Word data, int count, Word dupl)
 }
 
 
-static int
+static cmp_t
 compare_dict_entry(const void *a, const void *b, void *arg)
 { Word p = (Word)a+1;
   Word q = (Word)b+1;
@@ -240,10 +263,10 @@ compare_dict_entry(const void *a, const void *b, void *arg)
 }
 
 
-int
+bool
 dict_order(DECL_LD Word dict, Word dupl)
 { Functor data = (Functor)dict;
-  int arity = arityFunctor(data->definition);
+  size_t arity = arityFunctor(data->definition);
 
   assert(arity%2 == 1);
 
@@ -266,14 +289,15 @@ typedef struct order_term_refs
 
 
 #define compare_term_refs(ip1, ip2, ctx) LDFUNC(compare_term_refs, ip1, ip2, ctx)
-static inline int compare_term_refs(DECL_LD const int *ip1, const int *ip2, order_term_refs *ctx);
+static inline cmp_t compare_term_refs(DECL_LD const int *ip1, const int *ip2,
+				      order_term_refs *ctx);
 
-static int
+static cmp_t
 (compare_term_refs)(const void *a, const void *b, void *arg)
 { return compare_term_refs(PASS_AS_LD(((order_term_refs*)arg)->ld) a, b, arg);
 }
 
-static inline int
+static inline cmp_t
 compare_term_refs(DECL_LD const int *ip1, const int *ip2, order_term_refs *ctx)
 { Word p = valTermRef(ctx->av[*ip1*2]);
   Word q = valTermRef(ctx->av[*ip2*2]);
@@ -285,7 +309,7 @@ compare_term_refs(DECL_LD const int *ip1, const int *ip2, order_term_refs *ctx)
 }
 
 
-int
+int	/* 0 or index of duplicate key */
 dict_order_term_refs(DECL_LD term_t *av, int *indexes, int count)
 { order_term_refs ctx;
 
@@ -311,7 +335,8 @@ dict_order_term_refs(DECL_LD term_t *av, int *indexes, int count)
 
 
 #define assign_in_dict(dp, val) LDFUNC(assign_in_dict, dp, val)
-static int
+
+static boolex_t
 assign_in_dict(DECL_LD Word dp, Word val)
 { deRef(val);
 
@@ -334,11 +359,13 @@ assign_in_dict(DECL_LD Word dp, Word val)
 }
 
 
-#define put_dict(dict, size, nv, new_dict) LDFUNC(put_dict, dict, size, nv, new_dict)
-static int
-put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
+#define put_dict(dict, size, nv, new_dict) \
+	LDFUNC(put_dict, dict, size, nv, new_dict)
+
+static boolex_t
+put_dict(DECL_LD word dict, size_t size, Word nv, word *new_dict)
 { Functor data = valueTerm(dict);
-  int arity = arityFunctor(data->definition);
+  size_t arity = arityFunctor(data->definition);
   Word new, out, in, in_end, nv_end;
   int modified = false;
 
@@ -360,7 +387,7 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
 
   while(in < in_end && nv < nv_end)
   { Word i_name, n_name;
-    int rc;
+    boolex_t rc;
 
     deRef2(in+1, i_name);
     deRef2(nv+1, n_name);
@@ -401,7 +428,7 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
   } else
   { while(nv < nv_end)
     { Word n_name;
-      int rc;
+      boolex_t rc;
 
       deRef2(nv+1, n_name);
       if ( (rc=assign_in_dict(out++, nv)) != true )
@@ -421,11 +448,81 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
 }
 
 
-#define del_dict(dict, key, new_dict) LDFUNC(del_dict, dict, key, new_dict)
-static int
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Push a new dict to the global stack   that  has the same keys as `dict`,
+but whose tag and values are all set to variables.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#define copy_keys_dict(dict, new_dict) \
+	LDFUNC(copy_keys_dict, dict, new_dict)
+
+static boolex_t
+copy_keys_dict(DECL_LD word dict, word *new_dict)
+{ Functor data = valueTerm(dict);
+  size_t arity = arityFunctor(data->definition);
+  Word new, out, in, in_end;
+
+  if ( gTop+1+arity > gMax )
+    return GLOBAL_OVERFLOW;
+
+  new    = gTop;
+  out    = new;
+  *out++ = data->definition;		/* copy C'dict functor */
+  setVar(*out++);			/* set tag to var */
+  in     = data->arguments+1;
+  in_end = in+arity-1;
+
+  while(in < in_end)
+  { Word i_name;
+
+    deRef2(in+1, i_name);
+    setVar(*out++);
+    *out++ = *i_name;
+    in += 2;
+  }
+
+  gTop = out;
+  *new_dict = consPtr(new, TAG_COMPOUND|STG_GLOBAL);
+
+  return true;
+}
+
+#define same_keys_dict(dict1, dict2) \
+	LDFUNC(same_keys_dict, dict1, dict2)
+
+static bool
+same_keys_dict(DECL_LD word dict1, word dict2)
+{ Functor data1 = valueTerm(dict1);
+  Functor data2 = valueTerm(dict2);
+  size_t arity = arityFunctor(data1->definition);
+
+  if ( data1->definition != data2->definition )
+    return false;
+
+  Word d1 = data1->arguments+1;
+  Word d2 = data2->arguments+1;
+  Word dend = d1+arity-1;
+
+  for(; d1 < dend; d1+=2, d2+=2)
+  { Word k1, k2;
+
+    deRef2(d1+1, k1);
+    deRef2(d2+1, k2);
+    if ( *k1 != *k2 )
+      return false;
+  }
+
+  return true;
+}
+
+
+#define del_dict(dict, key, new_dict) \
+	LDFUNC(del_dict, dict, key, new_dict)
+
+static boolex_t
 del_dict(DECL_LD word dict, word key, word *new_dict)
 { Functor data = valueTerm(dict);
-  int arity = arityFunctor(data->definition);
+  size_t arity = arityFunctor(data->definition);
   Word new, out, in, in_end;
 
   assert(arity%2 == 1);
@@ -459,13 +556,29 @@ del_dict(DECL_LD word dict, word key, word *new_dict)
 }
 
 
+#define unify_tags(t1, t2, flags)			\
+	LDFUNC(unify_tags, t1, t2, flags)
+
+static boolex_t
+unify_tags(DECL_LD Word t1, Word t2, int flags)
+{ deRef(t1);
+  deRef(t2);
+
+  if ( *t1 == ATOM_dyndict || *t2 == ATOM_dyndict )
+    return true;
+
+  return unify_ptrs(t1, t2, flags);
+}
+
 /* partial_unify_dict(dict1, dict2) unifies all common elements of two
    dicts.  It returns true on success, false on a failed unification
    and *_OVERFLOW on some memory overflow
 */
 
-#define partial_unify_dict(dict1, dict2) LDFUNC(partial_unify_dict, dict1, dict2)
-static int
+#define partial_unify_dict(dict1, dict2) \
+	LDFUNC(partial_unify_dict, dict1, dict2)
+
+static boolex_t
 partial_unify_dict(DECL_LD word dict1, word dict2)
 { Functor d1 = valueTerm(dict1);
   Functor d2 = valueTerm(dict2);
@@ -475,8 +588,8 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
   Word end2 = in2+arityFunctor(d2->definition);
   int rc;
 
-  /* unify the tages */
-  if ( (rc=unify_ptrs(in1, in2, ALLOW_RETCODE)) != true )
+  /* unify the tags */
+  if ( (rc=unify_tags(in1, in2, ALLOW_RETCODE)) != true )
     return rc;
 
   /* advance to first v+k entry */
@@ -503,8 +616,7 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
   return true;
 }
 
-
-/* select_dict() demands del to be a sub-dict of from and assigns
+/* unify_left_dict() demands del to be a sub-dict of from and assigns
    all remaining values in new.
 
    Note that unify_ptrs() can push data onto the global stack in
@@ -512,20 +624,21 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
    two pass process.
 */
 
-#define select_dict(del, from, new_dict) LDFUNC(select_dict, del, from, new_dict)
-static int
-select_dict(DECL_LD word del, word from, word *new_dict)
+#define unify_left_dict(del, from) \
+	LDFUNC(unify_left_dict, del, from)
+
+static boolex_t
+unify_left_dict(DECL_LD word del, word from)
 { Functor dd = valueTerm(del);
   Functor fd = valueTerm(from);
   Word din  = dd->arguments;
   Word fin  = fd->arguments;
   Word dend = din+arityFunctor(dd->definition);
   Word fend = fin+arityFunctor(fd->definition);
-  size_t left = 0;
   int rc;
 
   /* unify the tags */
-  if ( (rc=unify_ptrs(din, fin, ALLOW_RETCODE)) != true )
+  if ( (rc=unify_tags(din, fin, ALLOW_RETCODE)) != true )
     return rc;
 
   /* advance to first v+k entry */
@@ -538,7 +651,7 @@ select_dict(DECL_LD word del, word from, word *new_dict)
     deRef2(din+1, d);
     deRef2(fin+1, f);
 
-    if ( *d == *f )
+    if ( *d == *f )		/* same keys */
     { if ( (rc = unify_ptrs(din, fin, ALLOW_RETCODE)) != true )
 	return rc;
       din += 2;
@@ -547,53 +660,102 @@ select_dict(DECL_LD word del, word from, word *new_dict)
     { return false;
     } else
     { fin += 2;
-      left++;
     }
   }
-  if ( din < dend )
-    return false;
-  left += (fend-fin)/2;
 
-  if ( !new_dict )
-    return true;
+  return din == dend;
+}
 
-  if ( gTop+2+2*left <= gMax )
-  { Word out = gTop;
 
-    *new_dict = consPtr(out, TAG_COMPOUND|STG_GLOBAL);
+#define select_dict(del, from, new_dict) \
+	LDFUNC(select_dict, del, from, new_dict)
 
-    *out++ = dict_functor(left);
-    setVar(*out++);			/* tag for new dict */
+static boolex_t
+select_dict(DECL_LD word del, word from, word *new_dict)
+{ Functor dd = valueTerm(del);
+  Functor fd = valueTerm(from);
+  Word din  = dd->arguments;
+  Word fin  = fd->arguments;
+  Word dend = din+arityFunctor(dd->definition);
+  Word fend = fin+arityFunctor(fd->definition);
+  int buf[256];
+  bit_vector *keep = NULL;
+  boolex_t rc;
 
-    din = dd->arguments+1;
-    fin = fd->arguments+1;
+  /* unify the tags */
+  if ( (rc=unify_tags(din, fin, ALLOW_RETCODE)) != true )
+    return rc;
 
-    while(left > 0)
-    { Word d, f;
+  /* advance to first v+k entry */
+  din++;
+  fin++;
 
-      deRef2(din+1, d);
-      deRef2(fin+1, f);
-      if ( *d == *f )
-      { din += 2;
-	fin += 2;
-      } else
-      { *out++ = linkValI(fin);
-	*out++ = *f;
-	fin += 2;
-	left--;
+  for(size_t i=0; din < dend && fin < fend; i++, fin += 2)
+  { Word d, f;
+
+    deRef2(din+1, d);
+    deRef2(fin+1, f);
+
+    if ( *d == *f )		/* same keys */
+    { if ( (rc = unify_ptrs(din, fin, ALLOW_RETCODE)) != true )
+	return rc;
+      din += 2;
+    } else if ( *d < *f )
+    { return false;
+    } else
+    { if ( !keep )
+      { size_t entries = arityFunctor(fd->definition)/2;
+	size_t sz = sizeof_bitvector(entries);
+
+	if ( sz <= sizeof(buf) )
+	  keep = (bit_vector*)buf;
+	else if ( !(keep = malloc(sz)) )
+	  return MEMORY_OVERFLOW;
+	init_bitvector(keep, entries);
       }
+      set_bit(keep, i);
     }
-    gTop = out;
-
-    return true;
   }
 
-  return GLOBAL_OVERFLOW;
+  if ( din == dend )		/* unification succeeded */
+  { size_t nsize = (keep ? popcount_bitvector(keep) : 0) + (fend-fin)/2;
+
+    if ( gTop+nsize*2+2 <= gMax )
+    { Word out = gTop;
+
+      *new_dict = consPtr(out, TAG_COMPOUND|STG_GLOBAL);
+      *out++ = dict_functor(nsize);
+      *out++ = ATOM_dyndict;
+
+      Word fe = fin;
+      fin  = fd->arguments+1;
+      for(size_t i=0; fin < fend; i++, fin += 2)
+      { if ( fin >= fe || (keep && true_bit(keep, i)) )
+	{ Word f;
+
+	  deRef2(fin+1, f);
+	  *out++ = linkValI(fin);
+	  *out++ = *f;
+	}
+      }
+      gTop = out;
+      rc = true;
+    } else
+    { rc = GLOBAL_OVERFLOW;
+    }
+  } else
+  { rc = false;
+  }
+
+  if ( keep && keep != (bit_vector*)buf )
+    free(keep);
+
+  return rc;
 }
 
 
 #define get_name_ex(t, np) LDFUNC(get_name_ex, t, np)
-static int
+static bool
 get_name_ex(DECL_LD term_t t, Word np)
 { Word p = valTermRef(t);
 
@@ -608,8 +770,10 @@ get_name_ex(DECL_LD term_t t, Word np)
 }
 
 
-#define get_name_value(p, name, value, m, flags) LDFUNC(get_name_value, p, name, value, m, flags)
-static int
+#define get_name_value(p, name, value, m, flags) \
+	LDFUNC(get_name_value, p, name, value, m, flags)
+
+static bool
 get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
 { const char *type;
 
@@ -666,7 +830,7 @@ get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
 		 *	 FOREIGN SUPPORT	*
 		 *******************************/
 
-int
+bool
 PL_is_dict(DECL_LD term_t t)
 { Word p = valTermRef(t);
 
@@ -695,7 +859,7 @@ API_STUB(bool)
 /* Turn data into a dict if it is not already a dict.
  */
 
-static int
+static bool
 PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags)
 { GET_LD
   word dupl;
@@ -785,23 +949,24 @@ the standard order of terms.
 
 typedef struct cmp_dict_index_data
 { Word  data;
-  int  *indexes;
+  size_t *indexes;
   PL_local_data_t *ld;
 } cmp_dict_index_data;
 
 #define cmp_dict_index(ip1, ip2, ctx) LDFUNC(cmp_dict_index, ip1, ip2, ctx)
-static inline int cmp_dict_index(DECL_LD const int *ip1, const int *ip2, cmp_dict_index_data *ctx);
+static inline cmp_t cmp_dict_index(DECL_LD const int *ip1, const int *ip2,
+				   cmp_dict_index_data *ctx);
 
 static int
 (cmp_dict_index)(const void *a1, const void *a2, void *arg)
 { return cmp_dict_index(PASS_AS_LD(((cmp_dict_index_data*)arg)->ld) a1, a2, arg);
 }
 
-static inline int
+static inline cmp_t
 cmp_dict_index(DECL_LD const int *ip1, const int *ip2, cmp_dict_index_data *ctx)
 { Word p = &ctx->data[*ip1*2+1];
   Word q = &ctx->data[*ip2*2+1];
-  int rc;
+  cmp_t rc;
 
   deRef(p);
   deRef(q);
@@ -858,10 +1023,10 @@ pl_for_dict(DECL_LD term_t dict,
 	   void *closure,
 	   int flags)
 { term_t av = PL_new_term_refs(2);
-  int i, arity, pairs;
+  size_t i, arity, pairs;
   Word p = valTermRef(dict);
-  int index_buf[256];
-  int *indexes = NULL;
+  size_t index_buf[256];
+  size_t *indexes = NULL;
   int rc = 0;
 
   deRef(p);
@@ -873,7 +1038,7 @@ pl_for_dict(DECL_LD term_t dict,
 
     if ( pairs < 256 )
       indexes = index_buf;
-    else if ( !(indexes = malloc(pairs*sizeof(int))) )
+    else if ( !(indexes = malloc(pairs*sizeof(size_t))) )
       return PL_no_memory();
 
     for(i=0; i<pairs; i++)
@@ -883,12 +1048,12 @@ pl_for_dict(DECL_LD term_t dict,
     ctx.data = argTermP(*p,1);
     ctx.indexes = indexes;
 
-    sort_r(indexes, pairs, sizeof(int), cmp_dict_index, &ctx);
+    sort_r(indexes, pairs, sizeof(size_t), cmp_dict_index, &ctx);
   }
 
   for(i=0; i < pairs; )
   { Word p = valTermRef(dict);
-    int in;
+    size_t in;
 
     if ( indexes )
     { in = indexes[i]*2+1;
@@ -990,7 +1155,7 @@ fix_firstvars(Code start, Code end)
   }
 }
 
-static int
+static bool
 resortDictsInCodes(Code PC, Code end)
 {
   for( ; PC < end; PC = stepPC(PC) )
@@ -1007,12 +1172,13 @@ resortDictsInCodes(Code PC, Code end)
 	if ( fd->name == ATOM_dict &&
 	     fd->arity > 1 &&
 	     fd->arity%2 == 1 )
-	{ int f, fields = fd->arity/2;
+	{ size_t f, fields = fd->arity/2;
 	  kv_code kv_buf[KV_PREALOCATED];
 	  code c_buf[C_PREALLOCATED];
 	  kv_code *kv_pos;
 	  Code c_tmp;
 	  Code fields_start, fs;
+	  ssize_t h_void = 0;
 
 	  if ( fields <= KV_PREALOCATED )
 	    kv_pos = kv_buf;
@@ -1025,14 +1191,14 @@ resortDictsInCodes(Code PC, Code end)
 	    PC[0] = encode(H_VOID);
 	    PC[1] = encode(H_VOID);
 	  }
-	  PC = skipArgs(PC, 1);		/* skip the type */
+	  PC = skipArgs(PC, 1, &h_void); /* skip the type */
 	  fields_start = PC;
 
 	  for(f = 0; f < fields; f++)
 	  { Code PCv = PC;
 
 	    kv_pos[f].start = PC-fields_start;
-	    PC = skipArgs(PC, 1);	/* skip value */
+	    PC = skipArgs(PC, 1, &h_void); /* skip value */
 
 	    code op = fetchop(PC);
 	    switch(op)
@@ -1128,7 +1294,7 @@ right_arg:
     Word ea;
     word dupl;
 
-    if ( fd->name == ATOM_dict && fd->arity%2 == 1 &&
+    if ( fd->name == ATOM_dict && fd->arity > 1 && fd->arity%2 == 1 &&
 	 dict_ordered(&t->arguments[1], fd->arity/2, &dupl) == false )
     { DEBUG(MSG_DICT, Sdprintf("Re-ordering dict\n"));
       dict_order((Word)t, &dupl);
@@ -1171,18 +1337,7 @@ PRED_IMPL("is_dict", 1, is_dict, 0)
   Word p = valTermRef(A1);
 
   deRef(p);
-  if ( isTerm(*p) )
-  { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
-    //word dupl;
-
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 /*&&
-	 dict_ordered(f->arguments+1, fd->arity/2, &dupl) == true*/ )
-      return true;
-  }
-
-  return false;
+  return is_dict_term(*p);
 }
 
 
@@ -1194,12 +1349,8 @@ PRED_IMPL("is_dict", 2, is_dict, 0)
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
-    //word dupl;
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 /*&&
-	 dict_ordered(f->arguments+1, fd->arity/2,  &dupl) == true*/ )
+    if ( is_dict_functor(f->definition) )
       return unify_ptrs(&f->arguments[0], valTermRef(A2),
 			ALLOW_GC|ALLOW_SHIFT);
   }
@@ -1218,7 +1369,7 @@ which these pairs are enumerated is _undefined_.
 static foreign_t
 pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
 { PRED_LD
-  int i;
+  size_t i;
   word dict;
 
   switch( CTX_CNTRL )
@@ -1232,7 +1383,7 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
       if ( is_dict_key(*np) )
       { Word vp;
 
-	if ( (vp=dict_lookup_ptr(dict, *np)) )
+	if ( (vp=dict_lookup_ptr(dict, *np, NULL)) )
 	  return unify_ptrs(vp, valTermRef(A3), ALLOW_GC|ALLOW_SHIFT);
 
 	if ( ex )
@@ -1250,11 +1401,11 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
     }
     case FRG_REDO:
     { Functor f;
-      int arity;
+      size_t arity;
       fid_t fid;
       Word p;
 
-      i = (int)CTX_INT + 2;
+      i = CTX_INT + 2;
       p = valTermRef(A2);
       deRef(p);
       dict = *p;
@@ -1319,7 +1470,7 @@ PRED_IMPL("get_dict", 5, get_dict, 0)
   if ( !get_name_ex(A1, &key) ||
        !(*valTermRef(av+1) = key) ||
        !get_create_dict_ex(A2, dt) ||
-       !(vp=dict_lookup_ptr(*valTermRef(dt), key)) ||
+       !(vp=dict_lookup_ptr(*valTermRef(dt), key, NULL)) ||
        !unify_ptrs(vp, valTermRef(A3), ALLOW_GC|ALLOW_SHIFT) ||
        !PL_put_term(av+0, A5) )
     return false;
@@ -1408,7 +1559,7 @@ PRED_IMPL("dict_pairs", 3, dict_pairs, 0)
       ctx.head = PL_new_term_refs(2);
       ctx.tmp  = ctx.head+1;
 
-      if ( PL_get_arg(1, dict, ctx.tmp) &&
+      if ( _PL_get_arg(1, dict, ctx.tmp) &&
 	   PL_unify(ctx.tmp, A2) &&
 	   _PL_for_dict(dict, put_pair, &ctx, PL_FOR_DICT_SORTED) == 0 )
 	return PL_unify_nil_ex(ctx.tail);
@@ -1425,6 +1576,60 @@ PRED_IMPL("dict_pairs", 3, dict_pairs, 0)
   return false;
 }
 
+/** dict_same_keys(?Dict1, ?Dict2) is semidet.
+*/
+
+#define unify_dict_copy(t, dt, dict)		\
+	LDFUNC(unify_dict_copy, t, dt, dict)
+
+static bool
+unify_dict_copy(DECL_LD term_t t, term_t dt, word dict)
+{ term_t tmp = PL_new_term_ref(); /* safe, we can allocate 10 */
+  word copy;
+  int rc;
+
+  for(;;)
+  { if ( (rc=copy_keys_dict(dict, &copy)) == true )
+    { *valTermRef(tmp) = copy;
+      return PL_unify(tmp, t);
+    }
+
+    if ( !makeMoreStackSpace(rc, ALLOW_SHIFT|ALLOW_GC) )
+      return false;
+
+    Word p = valTermRef(dt);
+    deRef(p);
+    dict = *p;
+  }
+}
+
+static
+PRED_IMPL("dict_same_keys", 2, dict_same_keys, 0)
+{ PRED_LD
+  Word d1 = valTermRef(A1);
+  Word d2 = valTermRef(A2);
+
+  deRef(d1);
+  deRef(d2);
+
+  if ( is_dict_term(*d1) )
+  { if ( is_dict_term(*d2) )
+      return same_keys_dict(*d1, *d2);
+    else if ( canBind(*d2) )
+      return unify_dict_copy(A2, A1, *d1);
+    else
+      return PL_type_error("dict", A2);
+  } else if ( is_dict_term(*d2) )
+  { if ( canBind(*d1) )
+      return unify_dict_copy(A1, A2, *d2);
+    else
+      return PL_type_error("dict", A1);
+  }
+
+  return PL_type_error("dict", A1);
+}
+
+
 
 /** put_dict(+New, +DictIn, -DictOut)
 
@@ -1436,16 +1641,16 @@ static
 PRED_IMPL("put_dict", 3, put_dict, 0)
 { PRED_LD
   term_t dt;
-  fid_t fid = PL_open_foreign_frame();
 
-retry:
   if ( (dt = PL_new_term_refs(2)) &&
        get_create_dict_ex(A2, dt+0) &&
        get_create_dict_ex(A1, dt+1) )
-  { Functor f2 = valueTerm(*valTermRef(dt+1));
-    int arity = arityFunctor(f2->definition);
+  { retry:
+    Mark(fli_context->mark);
+    Functor f2 = valueTerm(*valTermRef(dt+1));
+    size_t arity = arityFunctor(f2->definition);
     word new;
-    int rc;
+    boolex_t rc;
 
     if ( (rc = put_dict(*valTermRef(dt+0),
 			arity/2, &f2->arguments[1],
@@ -1456,7 +1661,7 @@ retry:
       return PL_unify(A3, t);
     } else
     { assert(rc == GLOBAL_OVERFLOW);
-      PL_rewind_foreign_frame(fid);
+      Undo(fli_context->mark);
       if ( makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
 	goto retry;
     }
@@ -1470,18 +1675,20 @@ retry:
 True when Dict is a copy of Dict0 with Name Value added or replaced.
 */
 
-#define put_dict4(key, dict, value, newdict) LDFUNC(put_dict4, key, dict, value, newdict)
+#define put_dict4(key, dict, value, newdict) \
+	LDFUNC(put_dict4, key, dict, value, newdict)
+
 static foreign_t
 put_dict4(DECL_LD term_t key, term_t dict, term_t value, term_t newdict)
 { term_t dt = PL_new_term_refs(3);
   term_t av = dt+1;
-  fid_t fid = PL_open_foreign_frame();
 
-retry:
   if ( get_create_dict_ex(dict, dt) &&
        get_name_ex(key, valTermRef(av+1)) &&
        PL_put_term(av, value) )
-  { word new;
+  { retry:
+    Mark(fli_context->mark);
+    word new;
     int rc;
 
     if ( (rc = put_dict(*valTermRef(dt),
@@ -1492,7 +1699,7 @@ retry:
       return PL_unify(newdict, t);
     } else
     { if ( makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
-      { PL_rewind_foreign_frame(fid);
+      { Undo(fli_context->mark);
 	goto retry;
       }
     }
@@ -1519,49 +1726,20 @@ Backtrackable destructive assignment, similar to setarg/3.
 #define SETDICT_BACKTRACKABLE    0x1
 #define SETDICT_LINK		0x2
 
-#define setdict(key, dict, value, flags) LDFUNC(setdict, key, dict, value, flags)
-static int
-setdict(DECL_LD term_t key, term_t dict, term_t value, int flags)
+#define setdict(key, dict, value, flags) \
+	LDFUNC(setdict, key, dict, value, flags)
+
+static bool
+setdict(DECL_LD term_t key, term_t dict, term_t value, unsigned int flags)
 { word k, m;
-  Word val;
-
-retry:
-  val = valTermRef(value);
-  deRef(val);
-
-  if ( (flags&SETDICT_BACKTRACKABLE) )
-  { if ( !hasGlobalSpace(0) )
-    { int rc;
-
-      if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != true )
-	return raiseStackOverflow(rc);
-      goto retry;
-    }
-  } else
-  { if ( storage(*val) == STG_GLOBAL )
-    { if ( !(flags & SETDICT_LINK) )
-      { term_t copy = PL_new_term_ref();
-
-	if ( !duplicate_term(value, copy) )
-	  return false;
-	value = copy;
-	val = valTermRef(value);
-	deRef(val);
-      }
-      freezeGlobal();
-    }
-  }
 
   if ( get_dict_ex(dict, &m, true) &&
        get_name_ex(key, &k) )
   { Word vp;
+    size_t arg;
 
-    if ( (vp=dict_lookup_ptr(m, k)) )
-    { if ( (flags&SETDICT_BACKTRACKABLE) )
-	TrailAssignment(vp);
-      unify_vp(vp, val);
-      return true;
-    }
+    if ( (vp=dict_lookup_ptr(m, k, &arg)) )
+      return setarg(arg*2, dict, value, flags);
 
     return PL_error(NULL, 0, NULL, ERR_EXISTENCE3,
 		    ATOM_key, key, dict);
@@ -1575,7 +1753,7 @@ static
 PRED_IMPL("b_set_dict", 3, b_set_dict, 0)
 { PRED_LD
 
-  return setdict(A1, A2, A3, SETDICT_BACKTRACKABLE);
+  return setdict(A1, A2, A3, SETARG_BACKTRACKABLE);
 }
 
 static
@@ -1589,7 +1767,7 @@ static
 PRED_IMPL("nb_link_dict", 3, nb_link_dict, 0)
 { PRED_LD
 
-  return setdict(A1, A2, A3, SETDICT_LINK);
+  return setdict(A1, A2, A3, SETARG_LINK);
 }
 
 
@@ -1611,7 +1789,7 @@ retry:
        get_name_ex(A1, &key) )
   { Word vp;
 
-    if ( (vp=dict_lookup_ptr(*valTermRef(mt), key)) &&
+    if ( (vp=dict_lookup_ptr(*valTermRef(mt), key, NULL)) &&
 	 unify_ptrs(vp, valTermRef(A3), ALLOW_GC|ALLOW_SHIFT) )
     { int rc;
       word new;
@@ -1648,7 +1826,8 @@ PRED_IMPL("select_dict", 3, select_dict, 0)
 retry:
   if ( get_create_dict_ex(A1, dt+0) &&
        get_create_dict_ex(A2, dt+1) )
-  { int rc = select_dict(*valTermRef(dt+0), *valTermRef(dt+1), &r);
+  { Mark(fli_context->mark);
+    int rc = select_dict(*valTermRef(dt+0), *valTermRef(dt+1), &r);
 
     switch(rc)
     { case true:
@@ -1662,6 +1841,7 @@ retry:
       case MEMORY_OVERFLOW:
 	return PL_no_memory();
       default:
+	Undo(fli_context->mark);
 	if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
 	  return false;
         goto retry;
@@ -1673,14 +1853,15 @@ retry:
 
 
 static
-PRED_IMPL(":<", 2, select_dict, 0)
+PRED_IMPL(":<", 2, unify_left_dict, 0)
 { PRED_LD
   term_t dt = PL_new_term_refs(2);
 
 retry:
   if ( get_create_dict_ex(A1, dt+0) &&
        get_create_dict_ex(A2, dt+1) )
-  { int rc = select_dict(*valTermRef(dt+0), *valTermRef(dt+1), NULL);
+  { Mark(fli_context->mark);
+    int rc = unify_left_dict(*valTermRef(dt+0), *valTermRef(dt+1));
 
     switch(rc)
     { case true:
@@ -1689,6 +1870,7 @@ retry:
       case MEMORY_OVERFLOW:
 	return PL_no_memory();
       default:
+	Undo(fli_context->mark);
 	if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
 	  return false;
         goto retry;
@@ -1725,6 +1907,87 @@ retry:
   return false;
 }
 
+/** '$get_dict_kv'(+Index, +Dict, -K, -V)
+    '$get_dict_kv'(+Index, +Dict1, +Dict2, -K, -V1, -V2)
+*/
+
+#define get_dict_kv(t0, arity)			\
+	LDFUNC(get_dict_kv, t0, arity)
+
+static bool
+get_dict_kv(DECL_LD term_t t0, int arity)
+{ size_t i;
+
+  if ( !PL_get_size_ex(t0, &i) || i == 0 )
+    return false;
+
+  Word p = valTermRef(t0+1);
+  deRef(p);
+  if ( isTerm(*p) )
+  { Functor f = valueTerm(*p);
+    size_t darity = arityFunctor(f->definition);
+
+    if ( !is_dict_functor(f->definition) )
+      return PL_type_error("dict", t0+1);
+
+    if ( i > darity/2 )
+      return false;
+    i--;
+
+    size_t dict_count = (arity-2)/2;
+    size_t key_arg = t0+1+dict_count;
+    term_t value = PL_new_term_ref();
+    term_t key = PL_new_term_ref();
+
+    for(size_t di=0; di < dict_count; di++)
+    { term_t dict = t0+1+di;
+
+      if ( di > 0 && !PL_is_functor(dict, f->definition) )
+      { if ( !PL_is_dict(dict) )
+	  return PL_type_error("dict", dict);
+	else
+	  return PL_domain_error("compatible_dict", dict);
+      }
+      _PL_get_arg(2+i*2, dict, value);
+      _PL_get_arg(3+i*2, dict, key);
+
+      if ( !PL_unify(key_arg, key) )
+      { if ( di == 0 )
+	  return false;
+	return PL_domain_error("compatible_dict", dict);
+      }
+
+      if ( !PL_unify(key_arg+1+di, value) )
+	return false;
+    }
+
+    return true;
+  }
+
+  return PL_type_error("dict", t0+1);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 4, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 4);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 6, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 6);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 8, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 8);
+}
+
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Part of FLI
@@ -1742,7 +2005,7 @@ PL_get_dict_key(atom_t key, term_t dict, term_t value)
 
   if ( !get_dict_ex(dict, &d, false) )
     return false;
-  if ( (vp=dict_lookup_ptr(d, key)) )
+  if ( (vp=dict_lookup_ptr(d, key, NULL)) )
   { *valTermRef(value) = linkValI(vp);
     return true;
   }
@@ -1756,20 +2019,24 @@ PL_get_dict_key(atom_t key, term_t dict, term_t value)
 		 *******************************/
 
 BeginPredDefs(dict)
-  PRED_DEF("is_dict",	   1, is_dict,	    0)
-  PRED_DEF("is_dict",	   2, is_dict,	    0)
-  PRED_DEF("dict_create",  3, dict_create,  0)
-  PRED_DEF("dict_pairs",   3, dict_pairs,   0)
-  PRED_DEF("put_dict",	   3, put_dict,	    0)
-  PRED_DEF("put_dict",	   4, put_dict,	    0)
-  PRED_DEF("b_set_dict",   3, b_set_dict,   0)
-  PRED_DEF("nb_set_dict",  3, nb_set_dict,  0)
-  PRED_DEF("nb_link_dict", 3, nb_link_dict, 0)
-  PRED_DEF("get_dict",	   3, get_dict,	    PL_FA_NONDETERMINISTIC)
-  PRED_DEF("$get_dict_ex", 3, get_dict_ex,  PL_FA_NONDETERMINISTIC)
-  PRED_DEF("del_dict",	   4, del_dict,	    0)
-  PRED_DEF("get_dict",     5, get_dict,     0)
-  PRED_DEF("select_dict",  3, select_dict,  0)
-  PRED_DEF(":<",	   2, select_dict,  0)
-  PRED_DEF(">:<",	   2, punify_dict,  0)
+  PRED_DEF("is_dict",	     1, is_dict,	0)
+  PRED_DEF("is_dict",	     2, is_dict,	0)
+  PRED_DEF("dict_create",    3, dict_create,    0)
+  PRED_DEF("dict_pairs",     3, dict_pairs,     0)
+  PRED_DEF("dict_same_keys", 2, dict_same_keys, 0)
+  PRED_DEF("put_dict",	     3, put_dict,       0)
+  PRED_DEF("put_dict",	     4, put_dict,       0)
+  PRED_DEF("b_set_dict",     3, b_set_dict,     0)
+  PRED_DEF("nb_set_dict",    3, nb_set_dict,    0)
+  PRED_DEF("nb_link_dict",   3, nb_link_dict,   0)
+  PRED_DEF("get_dict",	     3, get_dict,	PL_FA_NONDETERMINISTIC)
+  PRED_DEF("$get_dict_ex",   3, get_dict_ex,    PL_FA_NONDETERMINISTIC)
+  PRED_DEF("del_dict",	     4, del_dict,	0)
+  PRED_DEF("get_dict",       5, get_dict,       0)
+  PRED_DEF("select_dict",    3, select_dict,    0)
+  PRED_DEF(":<",	     2, unify_left_dict,0)
+  PRED_DEF(">:<",	     2, punify_dict,    0)
+  PRED_DEF("$get_dict_kv",   4, get_dict_kv,    0)
+  PRED_DEF("$get_dict_kv",   6, get_dict_kv,    0)
+  PRED_DEF("$get_dict_kv",   8, get_dict_kv,    0)
 EndPredDefs
