@@ -51,7 +51,7 @@ throw(error(<Formal>, <SWI-Prolog>))
 #include <errno.h>
 #endif
 
-static int
+static bool
 put_name_arity(term_t t, functor_t f)
 { GET_LD
   FunctorDef fdef = valueFunctor(f);
@@ -94,7 +94,7 @@ rewrite_callable(atom_t *expected, term_t actual)
 }
 
 
-static int
+static bool
 evaluation_error(term_t formal, atom_t which)
 { GET_LD
   return PL_unify_term(formal,
@@ -102,16 +102,18 @@ evaluation_error(term_t formal, atom_t which)
 			 PL_ATOM, which);
 }
 
-int
-PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
+bool
+PL_error(const char *pred, size_t arity, const char *msg, PL_error_code id, ...)
 { GET_LD
   char msgbuf[50];
   Definition caller;
   term_t except, formal, swi, msgterm=0;
   va_list args;
-  int do_throw = false;
   fid_t fid;
-  int rc;
+#if O_THROW
+  bool do_throw = false;
+#endif
+  bool rc;
   int msg_rep = REP_UTF8;
 
   if ( exception_term )			/* do not overrule older exception */
@@ -388,6 +390,18 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
 			   PL_CHARS, vmi);
       break;
     }
+    case ERR_PERMISSION_YIELD:
+    { term_t t;
+      rc = ( (t=PL_new_term_ref()) &&
+	     pl_thread_self(t) &&
+	     PL_unify_term(formal,
+			   PL_FUNCTOR, FUNCTOR_permission_error3,
+			     PL_ATOM, ATOM_yield,
+			     PL_ATOM, ATOM_engine,
+			     PL_TERM, t) );
+      PL_reset_term_refs(t);
+      break;
+    }
     case ERR_NOT_IMPLEMENTED_PROC:
     { const char *name = va_arg(args, const char *);
       int arity = va_arg(args, int);
@@ -720,7 +734,11 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
 			 PL_FUNCTOR, FUNCTOR_existence_error2,
 			   PL_ATOM, ATOM_stream,
 			   PL_POINTER, s);
+#if O_THROW
       do_throw = true;
+#else
+      PL_fatal_error("ERR_CLOSED_STREAM not supported without PL_throw()");
+#endif
       break;
     }
     case ERR_BUSY:
@@ -800,9 +818,11 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
     fatalError("Cannot report error: no memory");
   }
 
+#if O_THROW
   if ( do_throw )
     rc = PL_throw(except);
   else
+#endif
     rc = PL_raise_exception(except);
 
   PL_close_foreign_frame(fid);
@@ -891,7 +911,7 @@ PL_resource_error(const char *resource)
 }
 
 
-int
+bool
 PL_no_memory(void)
 { return PL_error(NULL, 0, NULL, ERR_RESOURCE, ATOM_memory);
 }
@@ -955,15 +975,26 @@ and true if the printing succeeded or merely failed.
 
 #define OK_RECURSIVE 10
 
-int
+bool
 printMessage(atom_t severity, ...)
+{ va_list args;
+  bool rc;
+
+  va_start(args, severity);
+  rc = printMessagev(severity, args);
+  va_end(args);
+
+  return rc;
+}
+
+bool
+printMessagev(atom_t severity, va_list args)
 { GET_LD
   valid_atom_t(severity);
   wakeup_state wstate;
   term_t av;
   predicate_t pred = PROCEDURE_print_message2;
-  va_list args;
-  int rc;
+  bool rc;
 
   if ( ++LD->in_print_message >= OK_RECURSIVE*3 )
     fatalError("printMessage(): recursive call\n");
@@ -973,25 +1004,36 @@ printMessage(atom_t severity, ...)
   }
 
   av = PL_new_term_refs(2);
-  va_start(args, severity);
   PL_put_atom(av+0, severity);
   rc = PL_unify_termv(av+1, args);
-  va_end(args);
 
   if ( rc )
-  { if ( isDefinedProcedure(pred) && LD->in_print_message <= OK_RECURSIVE )
-    { rc = PL_call_predicate(NULL, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
-			     pred, av);
-    } else if ( LD->in_print_message <= OK_RECURSIVE*2 )
-    { Sfprintf(Serror, "print_message/2: recursive call: ");
-      if ( ReadingSource )
-	Sfprintf(Serror, "%s:%d ",
-		 PL_atom_chars(source_file_name), (int)source_line_no);
-      rc = PL_write_term(Serror, av+1, 1200, 0);
-      Sfprintf(Serror, "\n");
-      PL_backtrace(5, 1);
-    } else				/* in_print_message == 2 */
-    { Sfprintf(Serror, "printMessage(): recursive call\n");
+  { if ( !Sferror(Suser_error) )
+    { if ( isDefinedProcedure(pred) &&
+	   LD->in_print_message <= OK_RECURSIVE )
+      { rc = PL_call_predicate(NULL, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
+			       pred, av);
+      } else
+      { if ( LD->in_print_message <= OK_RECURSIVE*2 )
+	{ Sdprintf("print_message/2: recursive call: ");
+	  if ( ReadingSource )
+	    Sdprintf("%s:%d ",
+		     PL_atom_chars(source_file_name), (int)source_line_no);
+	  rc = PL_write_term(Serror, av+1, 1200, 0);
+	  Sdprintf("\n");
+	  PL_backtrace(5, 1);
+	} else				/* in_print_message == 2 */
+	{ Sdprintf("printMessage(): recursive call\n");
+	}
+      }
+    } else
+    { if ( !Sferror(Serror) && truePrologFlag(PLFLAG_DEBUG_ON_ERROR) )
+      { Sdprintf("[%d] printMessage(): Cannot write to user_error:\n",
+		 PL_thread_self());
+	rc = PL_write_term(Serror, av+1, 1200, 0);
+	Sdprintf("\n");
+      } else
+	rc = false;
     }
   }
 
@@ -1011,7 +1053,7 @@ printMessage(atom_t severity, ...)
 		 *    ERROR-CHECKING *_get()	*
 		 *******************************/
 
-int
+bool
 PL_get_atom_ex(DECL_LD term_t t, atom_t *a)
 { if ( PL_get_atom(t, a) )
     succeed;
@@ -1041,10 +1083,10 @@ PL_get_integer_ex(term_t t, int *i)
   return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t);
 }
 
-int
+bool
 PL_get_long_ex(DECL_LD term_t t, long *i)
 { if ( PL_get_long(t, i) )
-    succeed;
+    return true;
 
   if ( PL_is_integer(t) )
     return PL_error(NULL, 0, NULL, ERR_REPRESENTATION, ATOM_long);
@@ -1059,12 +1101,9 @@ API_STUB(bool)
 )
 
 bool
-PL_get_int64_ex(term_t t, int64_t *i)
-{ GET_LD
-  valid_term_t(t);
-
-  if ( PL_get_int64(t, i) )
-    succeed;
+PL_get_int64_ex(DECL_LD term_t t, int64_t *i)
+{ if ( PL_get_int64(t, i) )
+    return true;
 
   if ( PL_is_integer(t) )
     return PL_error(NULL, 0, NULL, ERR_REPRESENTATION, ATOM_int64_t);
@@ -1072,8 +1111,13 @@ PL_get_int64_ex(term_t t, int64_t *i)
   return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t);
 }
 
+API_STUB(bool)
+(PL_get_int64_ex)(term_t t, int64_t *i)
+( valid_term_t(t);
+  return PL_get_int64_ex(t, i);
+)
 
-int
+bool
 PL_get_intptr_ex(DECL_LD term_t t, intptr_t *i)
 {
 #if SIZEOF_LONG != SIZEOF_VOIDP && SIZEOF_VOIDP == 8
@@ -1104,7 +1148,7 @@ fits_size(int64_t val)
 #define fits_size(v) true
 #endif
 
-int
+bool
 PL_get_size_ex(DECL_LD term_t t, size_t *i)
 { number n;
   Word p = valTermRef(t);
@@ -1169,7 +1213,7 @@ PL_get_size_ex(DECL_LD term_t t, size_t *i)
 }
 
 
-int
+bool
 pl_get_uint64(DECL_LD term_t t, uint64_t *i, int ex)
 { number n;
   Word p = valTermRef(t);
@@ -1228,7 +1272,7 @@ pl_get_uint64(DECL_LD term_t t, uint64_t *i, int ex)
   return ex ? PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t) : false;
 }
 
-int
+bool
 PL_get_uint64_ex(DECL_LD term_t t, uint64_t *i)
 { return pl_get_uint64(t, i, true);
 }
@@ -1349,31 +1393,31 @@ PL_unify_bool_ex(term_t t, int val)
 }
 
 
-int
-PL_get_arg_ex(int n, term_t term, term_t arg)
+bool
+PL_get_arg_ex(size_t n, term_t term, term_t arg)
 { GET_LD
   valid_term_t(term);
   valid_term_t(arg);
 
   if ( PL_get_arg(n, term, arg) )
-  { succeed;
+  { return true;
   } else
-  { term_t a = PL_new_term_ref();
+  { term_t a;
 
-    PL_put_integer(a, n);
-
-    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_natural, a);
+    return ( (a=PL_new_term_ref()) &&
+	     PL_put_uint64(a, n) &&
+	     PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_natural, a) );
   }
 }
 
 
-int
+bool
 PL_get_module_ex(term_t name, Module *m)
 { valid_term_t(name);
   if ( !PL_get_module(name, m) )
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_atom, name);
 
-  succeed;
+  return true;
 }
 
 
