@@ -98,7 +98,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include <mcheck.h>
 #endif
 
-static int	usage(void);
+static bool	usage(void);
 static bool	giveVersionInfo(const char *a);
 static bool	vsysError(const char *errtype, const char *fm, va_list args);
 static const char* abi_version(void);
@@ -543,6 +543,7 @@ findHome(const char *symbols, int argc, const char **argv)
       int rc = check_home(home);
       if ( rc < 0 && !is_boot_session(argc, argv) )
 	warn_bad_home("WARNING: Invalid SWI-Prolog home directory ", home, rc);
+      Setenv("SWI_HOME_DIR", home);
       return home;
     } else
     { fatalError("--home option too long");
@@ -699,9 +700,7 @@ setupGNUEmacsInferiorMode(void)
   if ( ((s = Getenv("EMACS", envbuf, sizeof(envbuf))) && s[0]) ||
        ((s = Getenv("INSIDE_EMACS", envbuf, sizeof(envbuf))) && s[0]) ||
        ((s = Getenv("INFERIOR", envbuf, sizeof(envbuf))) && streq(s, "yes")) )
-  { GET_LD
-
-    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+  { PL_set_prolog_flag("tty_control", PL_BOOL, false);
     val = true;
 #ifdef __WINDOWS__
     Sinput->flags  |= SIO_ISATTY;
@@ -841,10 +840,12 @@ initDefaults(void)
   GD->signals.sig_alert      = SIG_ALERT;
 #endif
 
-  if ( systemDefaults.notty )
-    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
-  else
-    setPrologFlagMask(PLFLAG_TTY_CONTROL);
+  if ( !current_prolog_flag("tty_control") )
+  { if ( systemDefaults.notty )
+      clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+    else
+      setPrologFlagMask(PLFLAG_TTY_CONTROL);
+  }
 
   setPrologFlagMask(PLFLAG_DEBUGINFO);
   setPrologFlagMask(PLFLAG_GCTHREAD);
@@ -1018,11 +1019,8 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 	  return -1;
       } else if ( (rc=is_bool_opt(s, "tty", &b)) )
       { if ( rc == true )
-	{ if ( b )
-	    setPrologFlagMask(PLFLAG_TTY_CONTROL);
-	  else
-	    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
-	} else
+	  PL_set_prolog_flag("tty_control", PL_BOOL, b);
+	else
 	  return -1;
       } else if ( (rc=is_bool_opt(s, "pce", &b)) )
       { if ( rc == true )
@@ -1050,7 +1048,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 	long sig = strtol(optval, &e, 10);
 
 	if ( e > optval && *e == EOS && sig >= 0 && sig < 32 )
-	  GD->signals.sig_alert = sig;
+	  GD->signals.sig_alert = (int) sig;
 	else
 	  return -1;
 #endif
@@ -1098,7 +1096,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 
     if ( streq(s, "tty") )
     { Sdprintf("Warning: `-tty` is deprecated.  Please use `--no-tty`\n");
-      clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+      PL_set_prolog_flag("tty_control", PL_BOOL, false);
       continue;
     }
 
@@ -1194,7 +1192,7 @@ Find the resource database.
 #endif
 
 static zipper *
-openResourceDB(int is_hash_bang)
+openResourceDB(bool is_hash_bang)
 { zipper *rc;
   char tmp[PATH_MAX];
   char plp[PATH_MAX];
@@ -1327,11 +1325,11 @@ Steps:
 
 bool
 PL_initialise(int argc, char **argv)
-{ int is_hash_bang = false;
+{ bool is_hash_bang = false;
   const char *rcpath = "<none>";
 
   if ( GD->initialised )
-    succeed;
+    return true;
 
 #ifdef O_DEBUG
   /* Initialize debug flag early, if first argument */
@@ -1352,7 +1350,7 @@ PL_initialise(int argc, char **argv)
   initDefaults();			/* Initialise global defaults */
   initPaths(argc, (const char**)argv);	/* fetch some useful paths */
 
-  { GET_LD
+  GET_LD
 #ifdef O_SIGNALS
     setPrologFlagMask(PLFLAG_SIGNALS);	/* default: handle signals */
 #endif
@@ -1436,12 +1434,21 @@ PL_initialise(int argc, char **argv)
   GD->cmdline.appl_argc = argc;
   GD->cmdline.appl_argv = argv;
 
+#ifdef __WINDOWS__
+  if ( truePrologFlag(PLFLAG_TTY_CONTROL) &&
+       !truePrologFlag(PLFLAG_EPILOG) )
+    PL_w32_wrap_ansi_console();	/* decode ANSI color sequences (ESC[...m) */
+  else
+    PL_set_prolog_flag("color_term", PL_BOOL, false);
+#endif
   setupGNUEmacsInferiorMode();		/* Detect running under EMACS */
 
   if ( !setupProlog() )
     return false;
-#ifdef O_PLMT
+#ifdef O_ENGINES
   aliasThread(PL_thread_self(), ATOM_thread, ATOM_main);
+#endif
+#ifdef O_PLMT
   enableThreads(!GD->options.nothreads);
 #endif
   PL_set_prolog_flag("resource_database", PL_ATOM|FF_READONLY, rcpath);
@@ -1517,7 +1524,6 @@ PL_initialise(int argc, char **argv)
   { int status = prologToplevel(PL_new_atom("$initialise"));
     return status;
   }
-  }					/* { GET_LD } */
 }
 
 
@@ -1532,7 +1538,7 @@ PL_set_resource_db_mem(const unsigned char *data, size_t size)
 
 typedef const char *cline;
 
-static int
+static bool
 usage(void)
 { char tmp[PATH_MAX];
 
@@ -1759,7 +1765,7 @@ PL_cleanup(int status)
   int asked_reclaim_memory = (status&PL_CLEANUP_NO_RECLAIM_MEMORY) == 0;
   int reclaim_memory = asked_reclaim_memory;
 
-  if ( GD->cleaning != CLN_NORMAL )
+  if ( GD->halt.cleaning != CLN_NORMAL )
     return PL_CLEANUP_RECURSIVE;
 
   checkPrologFlagsAccess();
@@ -1770,7 +1776,7 @@ PL_cleanup(int status)
 #endif
 
   PL_LOCK(L_INIT);
-  if ( GD->cleaning != CLN_NORMAL )
+  if ( GD->halt.cleaning != CLN_NORMAL )
   { PL_UNLOCK(L_INIT);
     return PL_CLEANUP_RECURSIVE;
   }
@@ -1786,7 +1792,7 @@ PL_cleanup(int status)
   GET_LD
 #endif
 
-  GD->cleaning = CLN_PROLOG;
+  GD->halt.cleaning = CLN_PROLOG;
   debugmode(false, NULL);		/* avoid recursive tracing */
 
   if ( GD->initialised )
@@ -1798,18 +1804,18 @@ PL_cleanup(int status)
     PL_set_prolog_flag("exit_status", PL_INTEGER, (intptr_t)rval);
     if ( query_loop(PL_new_atom("$run_at_halt"), false) == false &&
 	 !(status&PL_CLEANUP_NO_CANCEL) )
-    { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
-      { GD->cleaning = CLN_NORMAL;
+    { if ( ++GD->halt.cancelled	< MAX_HALT_CANCELLED )
+      { GD->halt.cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
 	return PL_CLEANUP_CANCELED;
       }
     }
 
-    GD->cleaning = CLN_FOREIGN;
+    GD->halt.cleaning = CLN_FOREIGN;
     if ( !run_on_halt(&GD->os.on_halt_list, rval) &&
 	 !(status&PL_CLEANUP_NO_CANCEL) )
-    { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
-      { GD->cleaning = CLN_NORMAL;
+    { if ( ++GD->halt.cancelled	< MAX_HALT_CANCELLED )
+      { GD->halt.cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
 	return PL_CLEANUP_CANCELED;
       }
@@ -1832,7 +1838,7 @@ PL_cleanup(int status)
 
 emergency:
 #endif
-  GD->cleaning = CLN_IO;
+  GD->halt.cleaning = CLN_IO;
 
   Scurout = Soutput;			/* reset output stream to user */
 
@@ -1840,7 +1846,7 @@ emergency:
   dieIO();				/* streams may refer to foreign code */
 					/* Standard I/O is only flushed! */
 
-  GD->cleaning = CLN_SHARED;
+  GD->halt.cleaning = CLN_SHARED;
 
   if ( GD->initialised )
   { fid_t cid = PL_open_foreign_frame();
@@ -1851,7 +1857,7 @@ emergency:
     PL_discard_foreign_frame(cid);
   }
 
-  GD->cleaning = CLN_DATA;
+  GD->halt.cleaning = CLN_DATA;
 
   RemoveTemporaryFiles();
 
@@ -2239,12 +2245,37 @@ vwarning(const char *fm, va_list args)
     }
   }
 
-  if ( !ReadingSource && truePrologFlag(PLFLAG_DEBUG_ON_ERROR) )
+  if ( !ReadingSource && truePrologFlag(PLFLAG_DEBUG_ON_ERROR) &&
+       validUserStreams() )
     pl_trace();
 
   return false;
 }
 
+static
+PRED_IMPL("$run_state", 1, run_state, 0)
+{ const char *rstate;
+
+  switch(GD->halt.cleaning)
+  { case CLN_NORMAL:
+      rstate = "normal";
+      break;
+    case CLN_PROLOG:
+      rstate = "cleanup_prolog";
+      break;
+    case CLN_FOREIGN:
+      rstate = "cleanup_foreign";
+      break;
+    case CLN_IO:
+    case CLN_SHARED:
+    case CLN_DATA:
+    default:
+      assert(0);
+      return false;
+  }
+
+  return PL_unify_atom_chars(A1, rstate);
+}
 
 
 
@@ -2253,5 +2284,6 @@ vwarning(const char *fm, va_list args)
 		 *******************************/
 
 BeginPredDefs(init)
-  PRED_DEF("$usage", 0, usage, 0)
+  PRED_DEF("$usage",     0, usage,     0)
+  PRED_DEF("$run_state", 1, run_state, 0)
 EndPredDefs
