@@ -114,7 +114,7 @@ firstRecordRecordList(RecordList rl)
 { RecordRef record;
 
   for(record = rl->firstRecord; record; record = record->next)
-  { if ( false(record->record, R_ERASED) )
+  { if ( isoff(record->record, R_ERASED) )
       return record;
   }
 
@@ -172,7 +172,7 @@ cleanRecordList(RecordList rl)
   for(r = rl->firstRecord; r; r = next )
   { next = r->next;
 
-    if ( true(r->record, R_ERASED) )
+    if ( ison(r->record, R_ERASED) )
       remove_record(r);
   }
 }
@@ -303,7 +303,7 @@ addUintBuffer(Buffer b, size_t val)
   { addBuffer(b, (uchar)val, uchar);
   } else
   { int zips = ((sizeof(val))*8+7-1)/7 - 1;
-    int leading = TRUE;
+    int leading = true;
 
     for(; zips >= 0; zips--)
     { uint d = (uint)((val >> zips*7) & 0x7f);
@@ -312,7 +312,7 @@ addUintBuffer(Buffer b, size_t val)
       { if ( zips != 0 )
 	  d |= 0x80;
 	addBuffer(b, (uchar)d, uchar);
-	leading = FALSE;
+	leading = false;
       }
     }
   }
@@ -417,7 +417,7 @@ addAtom(CompileInfo info, atom_t a)
   } else if ( unlikely(info->external) )
   { Atom ap = atomValue(a);
 
-    if ( true(ap->type, PL_BLOB_TEXT) )
+    if ( ison(ap->type, PL_BLOB_TEXT) )
     { if ( isUCSAtom(ap) )
 	addOpCode(info, PL_TYPE_EXT_WATOM);
       else
@@ -427,7 +427,7 @@ addAtom(CompileInfo info, atom_t a)
     } else
     { info->error = EFAST_SERIALIZE;
       info->econtext[0] = a;
-      return FALSE;
+      return false;
     }
   } else
   { addOpCode(info, PL_TYPE_ATOM);
@@ -436,7 +436,7 @@ addAtom(CompileInfo info, atom_t a)
       PL_register_atom(a);
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -457,7 +457,7 @@ addFunctor(CompileInfo info, functor_t f)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -533,7 +533,7 @@ compile_term_to_heap(DECL_LD term_agenda *agenda, CompileInfo info)
 	  DEBUG(9, Sdprintf("Added var-link %d\n", n));
 	} else
 	{ if ( !addAtom(info, word2atom(w)) )
-	    return FALSE;
+	    return false;
 	  DEBUG(9, Sdprintf("Added '%s'\n", stringAtom(w)));
 	}
 
@@ -594,7 +594,7 @@ compile_term_to_heap(DECL_LD term_agenda *agenda, CompileInfo info)
       }
       case TAG_COMPOUND:
       { Functor f = valueTerm(w);
-	int arity;
+	size_t arity;
 	functor_t functor;
 
 #if O_CYCLIC
@@ -615,7 +615,7 @@ compile_term_to_heap(DECL_LD term_agenda *agenda, CompileInfo info)
 	  mark.term = f;
 	  mark.fdef = functor;
 	  if ( !pushSegStack(&LD->cycle.lstack, mark, cycle_mark) )
-	    return FALSE;
+	    return false;
 	  f->definition = (functor_t)consUInt(info->size);
 				  /* overflow test (should not be possible) */
 	  DEBUG(CHK_SECURE, assert(valUInt(f->definition) == (uintptr_t)info->size));
@@ -624,13 +624,13 @@ compile_term_to_heap(DECL_LD term_agenda *agenda, CompileInfo info)
 
 	info->size += arity+1;
 	if ( !addFunctor(info, functor) )
-	  return FALSE;
+	  return false;
 	DEBUG(9, if ( GD->io_initialised )
 		   Sdprintf("Added %s/%d\n",
 			    stringAtom(valueFunctor(functor)->name),
 			    arityFunctor(functor)));
 	if ( !pushWorkAgenda(agenda, arity, f->arguments) )
-	  return FALSE;
+	  return false;
 	continue;
       }
       default:
@@ -638,7 +638,7 @@ compile_term_to_heap(DECL_LD term_agenda *agenda, CompileInfo info)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 #if USE_LD_MACROS
@@ -816,7 +816,7 @@ rec_error(CompileInfo info)
     }
     default:
       assert(0);
-      return FALSE;
+      return false;
   }
 }
 
@@ -836,27 +836,27 @@ compile_external_record(DECL_LD term_t t, record_data *data)
 
   init_cycle();
   initBuffer(&data->info.code);
-  data->info.external = TRUE;
-  data->info.lock = FALSE;
+  data->info.external = true;
+  data->info.lock = false;
 
   if ( get_int64(*p, &v) )		/* integer-only record */
   { first |= (REC_INT|REC_GROUND);
     addOpCode(&data->info, first);
     addInt64(&data->info, v);
-    data->simple = TRUE;
+    data->simple = true;
 
-    return TRUE;
+    return true;
   } else if ( isAtom(*p) )		/* atom-only record */
   { first |= (REC_ATOM|REC_GROUND);
     addOpCode(&data->info, first);
     if ( !addAtom(&data->info, word2atom(*p)) )
-      return FALSE;
-    data->simple = TRUE;
+      return false;
+    data->simple = true;
 
-    return TRUE;
+    return true;
   }
 					/* the real stuff */
-  data->simple = FALSE;
+  data->simple = false;
   initBuffer(&data->info.vars);
   data->info.size = 0;
   data->info.nvars = 0;
@@ -878,7 +878,7 @@ compile_external_record(DECL_LD term_t t, record_data *data)
   addUintBuffer((Buffer)&data->hdr, data->info.size);	/* size on stack */
   if ( data->info.nvars > 0 )
     addUintBuffer((Buffer)&data->hdr, data->info.nvars);/* Number of variables */
-  return TRUE;
+  return true;
 }
 
 
@@ -974,11 +974,11 @@ PRED_IMPL("fast_term_serialized", 2, fast_term_serialized, 0)
 	  return rc;
 	} else
 	{ discard_record_data(&data);
-	  return FALSE;
+	  return false;
 	}
       }
     } else
-    { return FALSE;
+    { return false;
     }
   } else if ( PL_get_nchars(string, &len, &rec,
 			    CVT_STRING|BUF_STACK|REP_ISO_LATIN_1|CVT_EXCEPTION) )
@@ -989,7 +989,7 @@ PRED_IMPL("fast_term_serialized", 2, fast_term_serialized, 0)
 	     PL_recorded_external(rec, tmp) &&
 	     PL_unify(term, tmp) );
   } else
-  { return FALSE;
+  { return false;
   }
 }
 
@@ -1029,7 +1029,7 @@ PRED_IMPL("fast_write", 2, fast_write, 0)
     return PL_release_stream(out) && rc;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1106,7 +1106,7 @@ PRED_IMPL("fast_read", 2, fast_read, 0)
 	    if ( Sfread(&rec[2], 1, size, in) != size )
 	      rc = PL_syntax_error("fastrw_integer", in);
 	    else
-	      rc = TRUE;
+	      rc = true;
 	  } else
 	  { rc = PL_syntax_error("fastrw_integer", in);
 	  }
@@ -1133,7 +1133,7 @@ PRED_IMPL("fast_read", 2, fast_read, 0)
 	      if ( (np=readSizeInt(in, &rec[2], &bytes)) &&
 		   (rec = realloc_record(rec, &np, bytes)) &&
 		   Sfread(np, 1, bytes, in) == bytes )
-		rc = TRUE;
+		rc = true;
 	      else
 		rc = PL_syntax_error("fastrw_atom", in);
 	      break;
@@ -1155,7 +1155,7 @@ PRED_IMPL("fast_read", 2, fast_read, 0)
 	       ((m&REC_GROUND) || (np=readSizeInt(in, np, &nvars))) &&
 	       (rec = realloc_record(rec, &np, codes)) &&
 	       Sfread(np, 1, codes, in) == codes )
-	    rc = TRUE;
+	    rc = true;
 	  else
 	    rc = PL_syntax_error("fastrw_term", in);
 	  break;
@@ -1182,7 +1182,7 @@ PRED_IMPL("fast_read", 2, fast_read, 0)
     return PL_release_stream(in) && rc;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1207,8 +1207,8 @@ typedef struct
 
 static void skipSizeInt(CopyInfo b);
 
-static inline int
-init_copy_vars(copy_info *info, uint n)
+static inline int		/* true or MEMORY_OVERFLOW */
+init_copy_vars(copy_info *info, size_t n)
 { if ( n > 0 )
   { Word *p;
 
@@ -1223,7 +1223,7 @@ init_copy_vars(copy_info *info, uint n)
   { info->vars = NULL;
   }
 
-  return TRUE;
+  return true;
 }
 
 static inline void
@@ -1320,7 +1320,7 @@ fetchExtFloat(CopyInfo b, void *f)
 
 static void
 fetchAtom(DECL_LD CopyInfo b, atom_t *a)
-{ unsigned int len = fetchSizeInt(b);
+{ size_t len = fetchSizeInt(b);
 
   *a = lookupAtom(b->data, len);
 
@@ -1330,7 +1330,7 @@ fetchAtom(DECL_LD CopyInfo b, atom_t *a)
 
 static void
 fetchAtomW(CopyInfo b, atom_t *a)
-{ unsigned int len = fetchSizeInt(b);
+{ size_t len = fetchSizeInt(b);
 
   *a = lookupUCSAtom((const pl_wchar_t*)b->data, len/sizeof(pl_wchar_t));
 
@@ -1339,7 +1339,7 @@ fetchAtomW(CopyInfo b, atom_t *a)
 
 
 static void
-fetchChars(CopyInfo b, unsigned len, Word to)
+fetchChars(CopyInfo b, size_t len, Word to)
 { fetchMultipleBuf(b, (char *)to, len, char);
 }
 
@@ -1348,7 +1348,7 @@ fetchChars(CopyInfo b, unsigned len, Word to)
 static int
 copy_record(DECL_LD Word p, CopyInfo b)
 { term_agenda agenda;
-  int is_compound = FALSE;
+  int is_compound = false;
   int tag;
 
   do
@@ -1381,10 +1381,10 @@ copy_record(DECL_LD Word p, CopyInfo b)
       }
 #if O_ATTVAR
       case PL_TYPE_ATTVAR:
-      { intptr_t n = fetchSizeInt(b);
+      { size_t n = fetchSizeInt(b);
 
 	DEBUG(MSG_REC_ATTVAR,
-	      Sdprintf("Restore attvar %ld at %p\n", (long)n, &b->gstore[1]));
+	      Sdprintf("Restore attvar %zd at %p\n", n, &b->gstore[1]));
 	register_attvar(b->gstore);
 	b->gstore[1] = consPtr(&b->gstore[2], TAG_ATTVAR|STG_GLOBAL);
 	*p = makeRefG(&b->gstore[1]);
@@ -1463,7 +1463,7 @@ copy_record(DECL_LD Word p, CopyInfo b)
       }
       case PL_TYPE_STRING:
       { size_t lw, len = fetchSizeInt(b);
-	int pad;
+	size_t pad;
 	word hdr;
 
 	lw = (len+sizeof(word))/sizeof(word); /* see globalBlob() */
@@ -1478,7 +1478,7 @@ copy_record(DECL_LD Word p, CopyInfo b)
       }
 #ifdef O_CYCLIC
       case PL_REC_CYCLE:
-      { unsigned offset = fetchSizeInt(b);
+      { size_t offset = fetchSizeInt(b);
 	Word ct = b->gbase+offset;
 
 	*p = consPtr(ct, TAG_COMPOUND|STG_GLOBAL);
@@ -1486,7 +1486,7 @@ copy_record(DECL_LD Word p, CopyInfo b)
       }
 #endif
     { word fdef;
-      int arity;
+      size_t arity;
       case PL_TYPE_COMPOUND:
 
 	fdef = fetchWord(b);
@@ -1498,7 +1498,7 @@ copy_record(DECL_LD Word p, CopyInfo b)
 	p = b->gstore;
 	b->gstore += arity;
 	if ( !is_compound )
-	{ is_compound = TRUE;
+	{ is_compound = true;
 	  initTermAgenda(&agenda, arity, p);
 	} else
 	{ if ( !pushWorkAgenda(&agenda, arity, p) )
@@ -1548,7 +1548,7 @@ copy_record(DECL_LD Word p, CopyInfo b)
 	p = b->gstore;
 	b->gstore += 2;
 	if ( !is_compound )
-	{ is_compound = TRUE;
+	{ is_compound = true;
 	  initTermAgenda(&agenda, 2, p);
 	} else
 	{ if ( !pushWorkAgenda(&agenda, 2, p) )
@@ -1561,11 +1561,11 @@ copy_record(DECL_LD Word p, CopyInfo b)
     }
   } while ( is_compound && (p=nextTermAgendaNoDeRef(&agenda)) );
 
-  return TRUE;
+  return true;
 }
 
 
-int
+int				/* true or *_OVERFLOW */
 copyRecordToGlobal(DECL_LD term_t copy, Record r, int flags)
 { copy_info b;
   int rc;
@@ -1576,25 +1576,25 @@ copyRecordToGlobal(DECL_LD term_t copy, Record r, int flags)
   assert(r->magic == REC_MAGIC);
 #endif
   if ( !hasGlobalSpace(r->gsize) )
-  { if ( (rc=ensureGlobalSpace(r->gsize, flags)) != TRUE )
+  { if ( (rc=ensureGlobalSpace(r->gsize, flags)) != true )
       return rc;
   }
   b.base = b.data = dataRecord(r);
   b.gbase = b.gstore = gTop;
   b.version_map = NULL;
 
-  if ( (rc=init_copy_vars(&b, r->nvars)) == TRUE )
+  if ( (rc=init_copy_vars(&b, r->nvars)) == true )
   { gTop += r->gsize;
     rc = copy_record(valTermRef(copy), &b);
     free_copy_vars(&b);
   }
-  if ( rc != TRUE )
+  if ( rc != true )
     return rc;
 
   assert(b.gstore == gTop);
   DEBUG(CHK_SECURE, checkData(valTermRef(copy)));
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1640,7 +1640,7 @@ is_external(const char *rec, size_t len)
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1655,7 +1655,7 @@ recursion.   Other   options:   combine     into    copyRecordToGlobal()
 
 static void
 skipAtom(CopyInfo b)
-{ uint len = fetchSizeInt(b);
+{ size_t len = fetchSizeInt(b);
 
   b->data += len;
 }
@@ -1734,20 +1734,20 @@ scanAtomsRecord(CopyInfo b, void (*func)(atom_t a))
 	continue;
       }
       case PL_TYPE_STRING:
-      { uint len = fetchSizeInt(b);
+      { size_t len = fetchSizeInt(b);
 	b->data += len;
 	continue;
       }
       case PL_TYPE_COMPOUND:
       { word fdef = fetchWord(b);
-	int arity;
+	size_t arity;
 
 	arity = arityFunctor(fdef);
 	work += arity;
 	continue;
       }
       case PL_TYPE_EXT_COMPOUND:
-      { intptr_t arity = fetchSizeInt(b);
+      { size_t arity = fetchSizeInt(b);
 
 	skipAtom(b);
 	work += arity;
@@ -1799,11 +1799,11 @@ unregister_atom_rec(atom_t a)
 
 bool
 freeRecord(Record record)
-{ if ( true(record, R_DUPLICATE) && --record->references > 0 )
+{ if ( ison(record, R_DUPLICATE) && --record->references > 0 )
     succeed;
 
 #ifdef O_ATOMGC
-  if ( false(record, (R_EXTERNAL|R_NOLOCK)) )
+  if ( isoff(record, (R_EXTERNAL|R_NOLOCK)) )
   { copy_info ci;
 
     DEBUG(3, Sdprintf("freeRecord(%p)\n", record));
@@ -1833,7 +1833,7 @@ unallocRecordRef(RecordRef r)
 
 static void
 freeRecordRef(RecordRef r)
-{ int reclaim_now = false(r->record, R_DBREF);
+{ int reclaim_now = isoff(r->record, R_DBREF);
 
   freeRecord(r->record);
   if ( reclaim_now )
@@ -1847,13 +1847,13 @@ freeRecordRef(RecordRef r)
 		 *	 EXTERNAL RECORDS	*
 		 *******************************/
 
-int
+bool
 PL_recorded_external(const char *rec, term_t t)
 { GET_LD
   copy_info b;
-  uint gsize;
+  size_t gsize;
   uchar m;
-  int rc;
+  bool rc;
 
   b.base = b.data = rec;
   b.version_map = NULL;
@@ -1912,12 +1912,12 @@ PL_recorded_external(const char *rec, term_t t)
   skipSizeInt(&b);			/* code-size */
   gsize = fetchSizeInt(&b);
   if ( !(b.gbase = b.gstore = allocGlobal(gsize)) )
-    return FALSE;			/* global stack overflow */
+    return false;			/* global stack overflow */
   b.dicts = 0;
   if ( !(m & REC_GROUND) )
-  { uint nvars = fetchSizeInt(&b);
+  { size_t nvars = fetchSizeInt(&b);
 
-    if ( (rc=init_copy_vars(&b, nvars)) == TRUE )
+    if ( (rc=init_copy_vars(&b, nvars)) == true )
     { rc = copy_record(valTermRef(t), &b);
       free_copy_vars(&b);
     }
@@ -1925,7 +1925,7 @@ PL_recorded_external(const char *rec, term_t t)
   { rc = copy_record(valTermRef(t), &b);
   }
 
-  if ( rc != TRUE )
+  if ( rc != true )
     return raiseStackOverflow(rc);
 
   assert(b.gstore == gTop);
@@ -1934,14 +1934,14 @@ PL_recorded_external(const char *rec, term_t t)
     resortDictsInTerm(t);
   DEBUG(CHK_SECURE, checkData(valTermRef(t)));
 
-  return TRUE;
+  return true;
 }
 
 
-int
+bool
 PL_erase_external(char *rec)
 { PL_free(rec);
-  return TRUE;
+  return true;
 }
 
 
@@ -1998,7 +1998,7 @@ PRED_IMPL("current_key", 1, current_key, PL_FA_NONDETERMINISTIC)
       { e = newTableEnumWP(GD->recorded_db.record_lists);
 	break;
       } else if ( getKeyEx(A1, &k) &&
-		  isCurrentRecordList(k, TRUE) )
+		  isCurrentRecordList(k, true) )
 	succeed;
 
       fail;
@@ -2011,7 +2011,7 @@ PRED_IMPL("current_key", 1, current_key, PL_FA_NONDETERMINISTIC)
       freeTableEnum(e);
       /*FALLTHROUGH*/
     default:				/* fool gcc */
-      return TRUE;
+      return true;
   }
 
   if ( (fid = PL_open_foreign_frame()) )
@@ -2038,7 +2038,7 @@ PRED_IMPL("current_key", 1, current_key, PL_FA_NONDETERMINISTIC)
   }
 
   freeTableEnum(e);
-  return FALSE;
+  return false;
 }
 
 
@@ -2073,7 +2073,7 @@ PL_record_az(word k, term_t term, term_t ref, record_az az)
   if ( ref && !PL_unify_recref(ref, r) )
   { PL_erase(copy);
     freeHeap(r, sizeof(*r));
-    return FALSE;
+    return false;
   }
 
   PL_LOCK(L_RECORD);
@@ -2136,7 +2136,7 @@ save_state(recorded_state *state)
   } else
   { recorded_state *newstate = allocForeignState(sizeof(*state));
     memcpy(newstate, state, sizeof(*state));
-    newstate->saved = TRUE;
+    newstate->saved = true;
     return newstate;
   }
 }
@@ -2150,7 +2150,7 @@ free_state(recorded_state *state)
   if ( state->r )
   { RecordList rl = state->r->list;
 
-    if ( --rl->references == 0 && true(rl, RL_DIRTY) )
+    if ( --rl->references == 0 && ison(rl, RL_DIRTY) )
       cleanRecordList(rl);
   }
   if ( state->saved )
@@ -2170,11 +2170,11 @@ advance_state(recorded_state *state)
   { if ( !r->next )
     { RecordList rl = r->list;
 
-      if ( --rl->references == 0 && true(rl, RL_DIRTY) )
+      if ( --rl->references == 0 && ison(rl, RL_DIRTY) )
 	cleanRecordList(rl);
     }
     r = r->next;
-  } while ( r && true(r->record, R_ERASED) );
+  } while ( r && ison(r->record, R_ERASED) );
 
   state->r = r;
   return r;
@@ -2210,12 +2210,12 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
 	    else
 	      rc = PL_unify(term, copy);
 	  } else
-	    rc = FALSE;
+	    rc = false;
 	  PL_UNLOCK(L_RECORD);
 
 	  return rc;
 	}
-	return FALSE;
+	return false;
       }
 
       memset(state, 0, sizeof(*state));
@@ -2225,15 +2225,15 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
       } else if ( getKeyEx(key, &k) )
       { RecordList rl;
 
-	if ( !(rl = isCurrentRecordList(k, TRUE)) )
-	  return FALSE;
+	if ( !(rl = isCurrentRecordList(k, true)) )
+	  return false;
 	PL_LOCK(L_RECORD);
 	rl->references++;
 	state->r = rl->firstRecord;
-	if ( true(state->r->record, R_ERASED) )
+	if ( ison(state->r->record, R_ERASED) )
 	  advance_state(state);
       } else
-      { return FALSE;
+      { return false;
       }
       break;
     }
@@ -2255,7 +2255,7 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
 
   /* Now holding L_RECORD */
   if ( (fid = PL_open_foreign_frame()) )
-  { int answered = FALSE;
+  { int answered = false;
     term_t copy = 0;
 
     while( !answered )
@@ -2283,7 +2283,7 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
 	  continue;
 	}
 
-	answered = TRUE;
+	answered = true;
 
 	if ( record->next )
 	{ state->r = record->next;
@@ -2318,7 +2318,7 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
 	} else
 	{ free_state(state);
 	  PL_UNLOCK(L_RECORD);
-	  return TRUE;
+	  return true;
 	}
       }
 
@@ -2332,7 +2332,7 @@ PRED_IMPL("recorded", va, recorded, PL_FA_NONDETERMINISTIC)
   free_state(state);
   PL_UNLOCK(L_RECORD);
 
-  return FALSE;
+  return false;
 }
 
 
@@ -2357,18 +2357,18 @@ fixup_ssu(DECL_LD term_t in, term_t out)
     atom_t a;
 
     if ( !g )
-      return FALSE;
+      return false;
 
     _PL_get_arg(1, body, g);
     _PL_get_arg(2, body, body);
 
     if ( PL_get_atom(g, &a) && a == ATOM_cut )
     { if ( !PL_put_term(guard, --g) )
-	return FALSE;
+	return false;
       while(--g > guard)
       { if ( !PL_cons_functor(guard, FUNCTOR_comma2, g, guard) ||
 	     !PL_cons_functor(head, FUNCTOR_comma2, head, guard) )
-	  return FALSE;
+	  return false;
       }
 
       return PL_unify_term(out,
@@ -2396,18 +2396,18 @@ PRED_IMPL("instance", 2, instance, 0)
   term_t term = A2;
 
   if ( !(ptr=PL_get_dbref(ref, &type)) )
-    return FALSE;
+    return false;
 
   if ( type == DB_REF_CLAUSE )
   { ClauseRef cref = ptr;
     Clause clause = cref->value.clause;
     gen_t generation = generationFrame(environment_frame);
 
-    if ( true(clause, GOAL_CLAUSE) ||
+    if ( ison(clause, GOAL_CLAUSE) ||
 	 !visibleClause(clause, generation) )
-      return FALSE;
+      return false;
 
-    if ( true(clause, UNIT_CLAUSE) )
+    if ( ison(clause, UNIT_CLAUSE) )
     { term_t head = PL_new_term_ref();
 
       return ( decompile(clause, head, 0) &&
@@ -2415,7 +2415,7 @@ PRED_IMPL("instance", 2, instance, 0)
 			     PL_FUNCTOR, FUNCTOR_prove2,
 			       PL_TERM, head,
 			       PL_ATOM, ATOM_true) );
-    } else if ( true(clause, SSU_CHOICE_CLAUSE) )
+    } else if ( ison(clause, SSU_CHOICE_CLAUSE) )
     { term_t tmp = PL_new_term_ref();
 
       return ( decompile(clause, tmp, 0) &&
@@ -2427,11 +2427,11 @@ PRED_IMPL("instance", 2, instance, 0)
   { RecordRef rref = ptr;
     term_t t = PL_new_term_ref();
 
-    if ( copyRecordToGlobal(t, rref->record, ALLOW_GC) == TRUE )
+    if ( copyRecordToGlobal(t, rref->record, ALLOW_GC) == true )
       return PL_unify(term, t);
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -2445,18 +2445,18 @@ PRED_IMPL("erase", 1, erase, 0)
   term_t ref = A1;
 
   if ( !(ptr=PL_get_dbref(ref, &type)) )
-    return FALSE;
+    return false;
 
   if ( type == DB_REF_CLAUSE )
   { ClauseRef cref = ptr;
     Clause clause = cref->value.clause;
     Definition def = clause->predicate;
 
-    if ( !true(def, P_DYNAMIC) )
+    if ( !ison(def, P_DYNAMIC) )
       return PL_error("erase", 1, NULL, ERR_PERMISSION,
 		      ATOM_clause, ATOM_erase, ref);
 
-    return retractClauseDefinition(def, clause, TRUE);
+    return retractClauseDefinition(def, clause, true);
   } else
   { RecordRef r = ptr;
     int rc;
