@@ -192,10 +192,50 @@ PL_get_choice(term_t r, Choice *chp)
 
 #ifdef O_DEBUGGER
 
+/**
+ * True when `FR` is created through a meta-call
+ */
+
+static bool
+is_meta_call(const LocalFrame fr)
+{ const LocalFrame parent = fr->parent;
+
+  if ( parent && levelFrame(fr) == levelFrame(parent)+1 &&
+       isoff(fr->predicate, P_FOREIGN) )
+  { Clause cl = parent->clause->value.clause;
+    const Code pc = prevPC(cl, fr->programPointer);
+
+    if ( pc )
+    { switch(fetchop(pc))
+      { case I_CALL1:
+	case I_CALLATM:
+	case I_DEPARTATMV:
+	case I_CALLATMV:
+	case I_CALLM:
+	case I_CALLN:
+	  return true;
+      }
+    }
+  }
+
+  return false;
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-isDebugFrame(frame, port) is true if this call   must  be visible in the
-tracer. `No-debug' code has HIDE_CHILDS. Calls to  it must be visible if
-the parent is a debug frame.
+isDebugFrame(frame, port) is true if this  call must be visible in the
+tracer.  We  consider system  code and  user code.   System predicates
+have  the flag  `HIDE_CHILDS` set.   The original  idea was  that user
+calls are  always visible, as  system predicates directly  called from
+user predicates.  In addition, we may show
+
+  - User hooks into system predicates, i.e., multifile predicates
+    called by system predicates.
+  - Calls from system meta-predicates if the meta-predicate itself
+    is called directly from user code and this is the actual meta
+    call.
+
+Note that if we run in debug  mode there is no last call optimization,
+making this a lot simpler.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 bool
@@ -207,25 +247,32 @@ isDebugFrame(const LocalFrame FR, int port)
     return true;			/* user pred */
 
   LocalFrame parent = FR->parent;
-  if ( alltrue(FR->predicate, P_NOPROFILE|HIDE_CHILDS) &&
-       parent && alltrue(parent->predicate, P_NOPROFILE|HIDE_CHILDS) )
-  { DEBUG(MSG_TRACE_FRAME,
-	  Sdprintf("noprof system-system: [%u] %s; parent [%u] %s\n",
-		   levelFrame(FR), predicateName(FR->predicate),
-		   levelFrame(parent), predicateName(parent->predicate)));
-    return false;
-  }
+  if ( parent )
+  { if ( alltrue(FR->predicate, P_NOPROFILE|HIDE_CHILDS) &&
+	 alltrue(parent->predicate, P_NOPROFILE|HIDE_CHILDS) )
+    { DEBUG(MSG_TRACE_FRAME,
+	    Sdprintf("noprof system-system: [%u] %s; parent [%u] %s\n",
+		     levelFrame(FR), predicateName(FR->predicate),
+		     levelFrame(parent), predicateName(parent->predicate)));
+      return false;
+    }
 
-  for(;
-      parent && ison(parent->predicate, P_NOPROFILE);
-      parent=parent->parent)
-    ;
+    for(;
+	parent && ison(parent->predicate, P_NOPROFILE);
+	parent=parent->parent)
+      ;
+  }
 
   DEBUG(MSG_TRACE_FRAME,
 	{ if ( parent )
-	    Sdprintf("isDebugFrame(): FR [%u] %s; parent [%u] %s\n",
-		     levelFrame(FR), predicateName(FR->predicate),
-		     levelFrame(parent), predicateName(parent->predicate));
+	    Sdprintf("isDebugFrame(): FR [%u%s] %s; parent [%s] [%u%s] %s\n",
+		     levelFrame(FR),
+		     ison(FR->predicate, HIDE_CHILDS) ? "H" : "",
+		     predicateName(FR->predicate),
+		     ison(parent, FR_HIDE_CHILDS) ? "H" : "",
+		     levelFrame(parent),
+		     ison(parent->predicate, HIDE_CHILDS) ? "H" : "",
+		     predicateName(parent->predicate));
 	});
 
   if ( parent )
@@ -235,7 +282,7 @@ isDebugFrame(const LocalFrame FR, int port)
 	return true;			/* user calls system */
       return false;			/* system calls system */
     } else
-    { if ( isoff(parent, FR_HIDE_CHILDS) )
+    { if ( isoff(parent, FR_HIDE_CHILDS) && is_meta_call(FR) )
 	return true;
       return false;
     }
@@ -258,7 +305,7 @@ exitFromDebugger(const char *msg, int status)
     return PL_TRACE_ACTION_ABORT;
   }
 #endif
-  Sfprintf(Sdout, "%sexit (status 4)\n", msg);
+  Sfprintf(Sdout, "%sexit (status %d)\n", msg, status);
   PL_halt(status);
   return -1;
 }
@@ -279,9 +326,9 @@ static bool		hasAlternativesFrame(LocalFrame);
 static void		alternatives(Choice);
 static int		exceptionDetails(void);
 static int		listGoal(LocalFrame frame);
-static LocalFrame	frameAtLevel(LocalFrame frame, int at_depth,
+static LocalFrame	frameAtLevel(LocalFrame frame, ssize_t at_depth,
 				     bool interactive);
-static int		saveGoal(LocalFrame frame, int at_depth,
+static int		saveGoal(LocalFrame frame, ssize_t at_depth,
 				 bool interactive);
 static int		traceInterception(LocalFrame, Choice, int, Code);
 static int		traceAction(char *cmd,
@@ -443,9 +490,15 @@ tracePort(DECL_LD LocalFrame frame, Choice bfr, int port, Code PC)
 		 levelFrame(frame), portPrompt(port),
 		 predicateName(frame->predicate)));
 
-  if ( (!isDebugFrame(frame, port) && !SYSTEM_MODE) || /* hidden */
-       debugstatus.suspendTrace )		/* called back */
+  if ( debugstatus.suspendTrace )
+  { DEBUG(MSG_TRACE_PORT, Sdprintf("Suspended\n"));
     return PL_TRACE_ACTION_CONTINUE;
+  }
+
+  if ( (!isDebugFrame(frame, port) && !SYSTEM_MODE) )
+  { DEBUG(MSG_TRACE_PORT, Sdprintf("nodebug frame\n"));
+    return PL_TRACE_ACTION_CONTINUE;
+  }
 
   if ( port == EXCEPTION_PORT )		/* do not trace abort */
   { if ( classify_exception(LD->exception.pending) >= EXCEPT_ABORT )
@@ -557,6 +610,8 @@ out:
   { SAVE_PTRS();
     abortProlog();
     RESTORE_PTRS();
+  } else if ( action == PL_TRACE_ACTION_HALT )
+  { exitFromDebugger("Debugger: ", 0);
   }
 
   return clear_skip(port, frame, action);
@@ -669,7 +724,7 @@ static int
 traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 	    bool interactive)
 { GET_LD
-  int num_arg;				/* numeric argument */
+  ssize_t num_arg;			/* numeric argument */
   int def_arg = true;			/* arg is default */
   char *s;
 
@@ -780,7 +835,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		debugmode(DBG_OFF, NULL);
 		return PL_TRACE_ACTION_CONTINUE;
     case 'g':	FeedBack("goals\n");
-		PL_backtrace(def_arg ? 5 : num_arg, PL_BT_USER);
+		PL_backtrace(def_arg ? 5 : (int)num_arg, PL_BT_USER);
 		return PL_TRACE_ACTION_AGAIN;
     case 'A':	FeedBack("alternatives\n");
 		alternatives(bfr);
@@ -910,7 +965,7 @@ bool
 put_frame_goal(term_t goal, LocalFrame frame)
 { GET_LD
   Definition def = frame->predicate;
-  int argc = def->functor->arity;
+  size_t argc = def->functor->arity;
   term_t fref = consTermRef((Word)frame);
 
   if ( !PL_unify_functor(goal, def->functor->functor) )
@@ -1236,7 +1291,7 @@ listGoal(LocalFrame frame)
 
 
 static LocalFrame
-frameAtLevel(LocalFrame frame, int at_depth, bool interactive)
+frameAtLevel(LocalFrame frame, ssize_t at_depth, bool interactive)
 { GET_LD
 
   if ( at_depth )
@@ -1262,7 +1317,7 @@ frameAtLevel(LocalFrame frame, int at_depth, bool interactive)
 
 
 static int
-saveGoal(LocalFrame frame, int at_depth, bool interactive)
+saveGoal(LocalFrame frame, ssize_t at_depth, bool interactive)
 { GET_LD
   fid_t fid;
   int rc = false;
@@ -1408,16 +1463,17 @@ PL_backtrace_string(int depth, int flags)
 #define process_trace_action(frame, port, action, nodebugp)		\
   LDFUNC(process_trace_action, frame, port, action, nodebugp)
 
-static int
+static char
 process_trace_action(DECL_LD LocalFrame frame, int port,
 		     term_t action, bool *nodebugp)
 { atom_t a, name;
   size_t arity;
-  int rval;
+  char rval;
 
   if ( PL_get_atom(action, &a) )
   { if ( a == ATOM_continue || a == ATOM_creep )
-    { rval = PL_TRACE_ACTION_CONTINUE;
+    { debugstatus.tracing = true;
+      rval = PL_TRACE_ACTION_CONTINUE;
     } else if ( a == ATOM_nodebug )
     { rval = PL_TRACE_ACTION_CONTINUE;
       *nodebugp = true;
@@ -1446,6 +1502,8 @@ process_trace_action(DECL_LD LocalFrame frame, int port,
     { rval = PL_TRACE_ACTION_ABORT;
       if ( !PL_exception(0) )
 	abortProlog();
+    } else if ( a == ATOM_halt )
+    { rval = PL_TRACE_ACTION_HALT;
     } else
     { PL_warning("Unknown trace action: %s", stringAtom(a));
       rval = PL_TRACE_ACTION_CONTINUE;
@@ -1534,8 +1592,12 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
     { RESTORE_PTRS();
       rval = process_trace_action(frame, port, argv+3, &nodebug);
     } else if ( (ex=PL_exception(qid)) )
-    { if ( classify_exception(ex) == EXCEPT_ABORT )
+    { except_class exclass = classify_exception(ex);
+
+      if ( exclass == EXCEPT_ABORT )
       { rval = PL_TRACE_ACTION_ABORT;
+      } else if ( exclass == EXCEPT_HALT )
+      { rval = PL_TRACE_ACTION_HALT;
       } else
       { if ( printMessage(ATOM_error, PL_TERM, ex) )
 	{ nodebug = true;
@@ -1603,7 +1665,7 @@ is prepared to generalise this.
 See also PL_backtrace() and os/pl-cstack.c.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+bool
 PL_get_context(pl_context_t *c, int thread_id)
 { GET_LD
   (void)thread_id;
@@ -1626,7 +1688,7 @@ PL_get_context(pl_context_t *c, int thread_id)
 }
 
 
-int
+bool
 PL_step_context(pl_context_t *c)
 { if ( c->fr )
   { GET_LD
@@ -1644,11 +1706,11 @@ PL_step_context(pl_context_t *c)
     }
   }
 
-  return c->fr ? true : false;
+  return c->fr != NULL;
 }
 
 
-int
+int	/* number of bytes printed */
 PL_describe_context(pl_context_t *c, char *buf, size_t len)
 { LocalFrame fr;
 
@@ -1923,8 +1985,12 @@ again:
     action_a:
 		unblockSignal(sig);
 		abortProlog();
+#if O_THROW
 		if ( !safe )
 		  PL_rethrow();
+#else
+		PL_fatal_error("Could not abort\n");
+#endif
 		break;
     case 'b':	Sfprintf(Sdout, "break\n");
 		if ( safe )
@@ -2066,9 +2132,10 @@ tracemode(bool doit, bool *old)
 
   if ( debugstatus.tracing != doit )
   { debugstatus.tracing = doit;
-    return printMessage(ATOM_silent,
-			PL_FUNCTOR_CHARS, "trace_mode", 1,
-			  PL_ATOM, doit ? ATOM_on : ATOM_off);
+    return ( validUserStreams() &&
+	     printMessage(ATOM_silent,
+			    PL_FUNCTOR_CHARS, "trace_mode", 1,
+			    PL_ATOM, doit ? ATOM_on : ATOM_off) );
   }
   if ( doit )				/* make sure trace works inside skip */
   { debugstatus.skiplevel = SKIP_VERY_DEEP;
@@ -2080,7 +2147,7 @@ tracemode(bool doit, bool *old)
 }
 
 
-static int
+static bool
 have_space_for_debugging(void)
 { GET_LD
 
@@ -2170,9 +2237,10 @@ debugmode(debug_type doit, debug_type *old)
     }
     debugstatus.debugging = doit;
     updateAlerted(LD);
-    return printMessage(ATOM_silent,
-			PL_FUNCTOR_CHARS, "debug_mode", 1,
-			  PL_ATOM, doit ? ATOM_on : ATOM_off);
+    return ( validUserStreams() &&
+	     printMessage(ATOM_silent,
+			    PL_FUNCTOR_CHARS, "debug_mode", 1,
+			      PL_ATOM, doit ? ATOM_on : ATOM_off) );
   } else if ( !doit )
   { setPrologRunMode(RUN_MODE_NORMAL);
   }
@@ -2258,9 +2326,7 @@ pl_spy(term_t p)
   { Definition def = getProcDefinition(proc);
 
     if ( isoff(def, SPY_ME) )
-    { LOCKDEF(def);
-      set(def, SPY_ME);
-      UNLOCKDEF(def);
+    { set(def, SPY_ME);
       if ( !printMessage(ATOM_informational,
 			 PL_FUNCTOR_CHARS, "spy", 1,
 			   PL_TERM, p) )
@@ -2281,9 +2347,7 @@ pl_nospy(term_t p)
   { Definition def = getProcDefinition(proc);
 
     if ( ison(def, SPY_ME) )
-    { LOCKDEF(def);
-      clear(def, SPY_ME);
-      UNLOCKDEF(def);
+    { clear(def, SPY_ME);
       return printMessage(ATOM_informational,
 			  PL_FUNCTOR_CHARS, "nospy", 1,
 			    PL_TERM, p);
@@ -2400,7 +2464,7 @@ clear_frame_vars(LocalFrame target)
 }
 
 
-static int
+static bool
 prolog_frame_attribute(term_t frame, term_t what, term_t value)
 { GET_LD
   LocalFrame fr;
@@ -2420,7 +2484,7 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
   if ( !PL_strip_module(value, &m, value) )
     return false;
 
-  set(fr, FR_DEBUG);			/* explicit call to do this? */
+  set(fr, FR_NOTIFY);			/* explicit call to do this? */
 
   if ( key == ATOM_argument && arity == 1 )
   { term_t arg = PL_new_term_ref();
@@ -2445,7 +2509,7 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
 #endif
 
     if ( !hasGlobalSpace(0) )
-    { int rc;
+    { bool rc;
 
       if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != true )
 	return raiseStackOverflow(rc);
@@ -2520,7 +2584,7 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
 	}
 
 	if ( fr )
-	{ int i, garity = fr->predicate->functor->arity;
+	{ size_t i, garity = fr->predicate->functor->arity;
 
 	  fref = consTermRef((Word)fr);
 
@@ -2718,12 +2782,13 @@ bool
 PL_set_trace_action(term_t action)
 { GET_LD
   bool nodebug = false;
-  int rc = process_trace_action(LD->environment,
-				LD->trace.yield.port, action,
-				&nodebug);
+  char rc = process_trace_action(LD->environment,
+				 LD->trace.yield.port, action,
+				 &nodebug);
 
   if ( rc >= 0 )
-  { LD->trace.yield.resume_action = rc;
+  { clear_skip(LD->trace.yield.port, LD->environment, rc);
+    LD->trace.yield.resume_action = rc;
     LD->trace.yield.nodebug = nodebug;
     return true;
   }
