@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2016-2024, VU University Amsterdam
+    Copyright (c)  2016-2025, VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -49,6 +49,7 @@
 #include "pl-util.h"
 #include "pl-attvar.h"
 #include "pl-pro.h"
+#include "os/pl-buffer.h"
 #define NO_AC_TERM_WALK 1
 #define AC_TERM_WALK_POP 1
 #include "pl-termwalk.c"
@@ -95,28 +96,32 @@ typedef enum
 typedef struct ukey_state
 { trie	       *trie;			/* Trie for indirects */
   Word		ptr;			/* current location */
-  unify_mode	umode;			/* unification mode */
-  size_t	max_var_seen;
-  size_t	vars_allocated;		/* # variables allocated */
-  Word*		vars;
   size_t        a_offset;		/* For resetting the argument stack */
-  Word		var_buf[NVARS_FAST];	/* quick var buffer */
+  unify_mode	umode;			/* unification mode */
+  tmp_buffer	vars;
 } ukey_state;
 
+#if O_TRIE_ATTVAR
+typedef struct
+{ Word attvar;
+  Word value;
+} ukey_attvar;
+#endif
+
 #if USE_LD_MACROS
-#define	unify_key(state, key)			LDFUNC(unify_key, state, key)
-#define	init_ukey_state(state, trie, p)		LDFUNC(init_ukey_state, state, trie, p)
-#define	destroy_ukey_state(state)		LDFUNC(destroy_ukey_state, state)
+#define	unify_key(state, key)		LDFUNC(unify_key, state, key)
+#define	init_ukey_state(state, trie, p, reinit) \
+	LDFUNC(init_ukey_state, state, trie, p, reinit)
+#define	destroy_ukey_state(state)	LDFUNC(destroy_ukey_state, state)
 #endif /*USE_LD_MACROS*/
 
 #define LDFUNC_DECLARATIONS
-
 static int	unify_key(ukey_state *state, word key);
-static void	init_ukey_state(ukey_state *state, trie *trie, Word p);
+static void	init_ukey_state(ukey_state *state, trie *trie, Word p,
+				bool reinit);
 static void	destroy_ukey_state(ukey_state *state);
 static void	set_trie_clause_general_undefined(Clause cl);
 static void	trie_destroy(trie *trie);
-
 #undef LDFUNC_DECLARATIONS
 
 
@@ -240,7 +245,7 @@ trie_discard_clause(trie *trie)
 
   if ( (dbref=trie->clause) )
   { if ( COMPARE_AND_SWAP_ATOM(&trie->clause, dbref, 0) &&
-	 GD->cleaning == CLN_NORMAL )		/* otherwise reclaims clause */
+	 GD->halt.cleaning == CLN_NORMAL )		/* otherwise reclaims clause */
     { ClauseRef cref = clause_clref(dbref);	/* from two ends */
 
       if ( cref )
@@ -299,7 +304,7 @@ get_child(DECL_LD trie_node *n, word key)
 }
 
 
-int
+bool
 is_leaf_trie_node(trie_node *n)
 { trie_children children = n->children;
 
@@ -650,9 +655,11 @@ insert_child(DECL_LD trie *trie, trie_node *n, word key)
 }
 
 
-#define follow_node(trie, n, value, add) LDFUNC(follow_node, trie, n, value, add)
+#define follow_node(trie, n, value, add) \
+	LDFUNC(follow_node, trie, n, value, add)
+
 static trie_node *
-follow_node(DECL_LD trie *trie, trie_node *n, word value, int add)
+follow_node(DECL_LD trie *trie, trie_node *n, word value, bool add)
 { trie_node *child;
 
   if ( (child=get_child(n, value)) )
@@ -665,7 +672,9 @@ follow_node(DECL_LD trie *trie, trie_node *n, word value, int add)
 }
 
 
-#define trie_intern_indirect(trie, w, add) LDFUNC(trie_intern_indirect, trie, w, add)
+#define trie_intern_indirect(trie, w, add) \
+	LDFUNC(trie_intern_indirect, trie, w, add)
+
 static word
 trie_intern_indirect(DECL_LD trie *trie, word w, int add)
 { for(;;)
@@ -713,22 +722,56 @@ Return:
     Ok (found or inserted)
   - TRIE_ABSTRACTED
     Ok, but abstracted
-  - TRIE_LOOKUP_CONTAINS_ATTVAR
+  - TRIE_LOOKUP_CONTAINS_ATTVAR (unless O_TRIE_ATTVAR is activated)
   - TRIE_LOOKUP_CYCLIC
 
+# Variable handling
+
+When a fresh variable is encountered, we push its location onto `vars`
+and replace it with the variable number and the TAG_VAR.  Next time we
+find this variable we simply use the stored word.  Thus, f(X,X) is
+
+    <functor><var 1><var 1>POP
+
+In  theory, we  could do  singleton  detection and  only use  variable
+numbers for singletons.  The price is  though that this needs an extra
+pass.
+
+__Attributed__ variables are represented as  below, where the <N> uses
+the same  numbering as for variables.   When pushing to the  stack, we
+both both  push the  location of  the attvar and  the location  of the
+attributes.
+
+    <begin_attvar N> <attributes ...> <pop> <end_attvar N>
+
+<begin_attvar N> is (N<<LMASK_BITS)|TAG_ATTVAR|STG_STATIC
+<end_attvar N>   is (N<<LMASK_BITS)|TAG_ATTVAR|STG_RESERVED
+<pop>		 is TRIE_KEY_POP(popn), compensating for the push
+                    for writing the attributes and restoring read/write
+                    umode.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+typedef struct attvar_mark
+{ size_t compound_depth;
+  Word   attvar;
+} attvar_mark;
+
+int	/* bool|TRIE_ABSTRACTED|TRIE_LOOKUP_CONTAINS_ATTVAR|TRIE_LOOKUP_CYCLIC */
 trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
-		     Word k, int add, size_abstract *abstract,
+		     Word k, bool add, size_abstract *abstract,
 		     TmpBuffer vars)
 { term_agenda_P agenda;
   size_t var_number = 0;
   int rc = true;
-  size_t compounds = 0;
+  size_t compounds = 0;		/* Nesting depth */
   tmp_buffer varb;
   size_abstract sa = {.from_depth = 1, .size = (size_t)-1};
   size_t aleft = (size_t)-1;
+#if O_TRIE_ATTVAR
+  tmp_buffer attvarb;
+  TmpBuffer attvars = NULL;
+  attvar_mark *avm = NULL;
+#endif
 
   TRIE_STAT_INC(trie, lookups);
   if ( !node )
@@ -749,9 +792,32 @@ trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
       if ( compounds > 0 )
       { if ( !(node = follow_node(trie, node, TRIE_KEY_POP(popn), add)) )
 	  break;
+#if O_TRIE_ATTVAR
+	if ( avm && avm->compound_depth == compounds )
+	{ word w = (*avm->attvar)|STG_RESERVED;
+	  DEBUG(MSG_TRIE_PUT_TERM,
+		Sdprintf("Finished attvar at %p\n", avm->attvar));
+	  (void)popBufferP(attvars, attvar_mark);
+	  avm--;
+	  if ( !(node = follow_node(trie, node, w, add)) )
+	    break;
+	}
+#endif
 	continue;
       } else
+      {
+#if O_TRIE_ATTVAR
+	if ( avm && avm->compound_depth == compounds )
+	{ word w = (*avm->attvar)|STG_RESERVED;
+	  DEBUG(MSG_TRIE_PUT_TERM,
+		Sdprintf("Finished attvar at %p\n", avm->attvar));
+	  if ( !(node = follow_node(trie, node, TRIE_KEY_POP(popn), add)) )
+	    break;
+	  node = follow_node(trie, node, w, add);
+	}
+#endif
 	break;				/* finished toplevel */
+      }
     }
 
     if ( compounds == sa.from_depth )
@@ -770,18 +836,49 @@ trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
 	  }
 	  addBuffer(vars, p, Word);
 	  w2 = ((((word)var_number))<<LMASK_BITS)|TAG_VAR;
-	  if ( tag(w) == TAG_VAR )
+	  if ( tag(w) == TAG_VAR ) /* otherwise abstracted */
 	    *p = w2;
 	  w = w2;
 	}
         node = follow_node(trie, node, w, add);
 	break;
       case TAG_ATTVAR:
+#if O_TRIE_ATTVAR
+	if ( tagex(w) != (TAG_ATTVAR|STG_STATIC) )
+	{ Word ap = valPAttVar(w);
+	  if ( var_number++ == 0 && !vars )
+	  { vars = &varb;
+	    initBuffer(vars);
+	  }
+	  addBuffer(vars, p, Word);
+	  addBuffer(vars, ap, Word);
+	  w = ((((word)var_number))<<LMASK_BITS)|TAG_ATTVAR|STG_STATIC;
+	  *p = w;
+	  node = follow_node(trie, node, w, add);
+	  if ( !node )
+	    break;
+	  if ( !attvars )
+	  { attvars = &attvarb;
+	    initBuffer(attvars);
+	  }
+	  avm = allocFromBuffer(attvars, sizeof(*avm));
+	  avm->compound_depth = compounds;
+	  avm->attvar = p;
+	  DEBUG(MSG_TRIE_PUT_TERM,
+		Sdprintf("Opened attvar at %p\n", p));
+	  compounds++;
+	  pushWorkAgenda_P0(&agenda, 1, ap);
+	} else
+	{ DEBUG(MSG_TRIE_PUT_TERM,
+		Sdprintf("Shared attvar at %p\n", p));
+	  node = follow_node(trie, node, w, add);
+	}
+#else
 	rc = TRIE_LOOKUP_CONTAINS_ATTVAR;
-
-        prune_error(trie, node);
-        node = NULL;
-        break;
+	prune_error(trie, node);
+	node = NULL;
+#endif
+	break;
       case TAG_COMPOUND:
       { if ( unlikely(aleft == 0) )
 	{ rc = TRIE_ABSTRACTED;
@@ -826,10 +923,20 @@ trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
     for(; pp < ep; pp++)
     { Word vp = *pp;
       if ( tag(*vp) == TAG_VAR )
-	setVar(*vp);
+      { setVar(*vp);
+#if O_TRIE_ATTVAR
+      } else if ( tagex(*vp) == (TAG_ATTVAR|STG_STATIC) )
+      { Word ap = *++pp;
+	*vp = consPtr(ap, TAG_ATTVAR|STG_GLOBAL);
+#endif
+      }
     }
     if ( vars == &varb )
       discardBuffer(vars);
+#if O_TRIE_ATTVAR
+    if ( attvars == &attvarb )
+      discardBuffer(attvars);
+#endif
   }
 
   if ( rc > 0 )
@@ -839,6 +946,8 @@ trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
       rc = false;
   }
 
+  DEBUG(CHK_SECURE, checkStacks(NULL));
+
   return rc;
 }
 
@@ -856,10 +965,14 @@ get_trie_from_node(trie_node *node)
 }
 
 
-int
+bool
 is_ground_trie_node(trie_node *node)
 { for( ; node->parent; node = node->parent )
-  { if ( tagex(node->key) == TAG_VAR )
+  { if ( tagex(node->key) == TAG_VAR
+#if O_TRIE_ATTVAR
+	 || tagex(node->key) == (TAG_ATTVAR|STG_STATIC)
+#endif
+       )
       return false;
   }
 
@@ -901,13 +1014,13 @@ print_keys(Word k, size_t kc)
 
 #define MAX_FAST 256
 
-int
+bool
 unify_trie_term(DECL_LD trie_node *node, trie_node **parent, term_t term)
 { word fast[MAX_FAST];
   Word keys = fast;
   size_t keys_allocated = MAX_FAST;
   size_t kc = 0;
-  int rc = true;
+  bool rc = true;
   trie *trie_ptr;
   mark m;
   int is_secondary = ison(node, TN_SECONDARY);
@@ -946,21 +1059,31 @@ unify_trie_term(DECL_LD trie_node *node, trie_node **parent, term_t term)
   { ukey_state ustate;
     size_t i;
 
+    init_ukey_state(&ustate, trie_ptr, valTermRef(term), false);
   retry:
     Mark(m);
-    init_ukey_state(&ustate, trie_ptr, valTermRef(term));
+    int rcu = true;
     for(i=kc; i-- > 0; )
-    { if ( (rc=unify_key(&ustate, keys[i])) != true )
-      { destroy_ukey_state(&ustate);
-	if ( rc == false )
-	  goto out;
-	Undo(m);
-	if ( (rc=makeMoreStackSpace(rc, ALLOW_GC)) )
-	  goto retry;
-	else
-	  goto out;
-      }
+    { rcu = unify_key(&ustate, keys[i]);
+
+      if ( rcu != true )
+	break;
     }
+
+    if ( rcu != true )
+    { if ( rcu == false )
+      { destroy_ukey_state(&ustate);
+	rc = false;
+	goto out;
+      }
+      Undo(m);
+      if ( (rc=makeMoreStackSpace(rcu, ALLOW_GC)) )
+      { init_ukey_state(&ustate, trie_ptr, valTermRef(term), true);
+	goto retry;
+      } else
+	goto out;
+    }
+
     destroy_ukey_state(&ustate);
     break;
   }
@@ -969,6 +1092,7 @@ out:
   if ( keys != fast )
     free(keys);
 
+  DEBUG(CHK_SECURE, checkStacks(NULL));
   return rc;
 }
 
@@ -1103,12 +1227,12 @@ symbol_trie(atom_t symbol)
 
 
 #define unify_trie(t, trie) LDFUNC(unify_trie, t, trie)
-static int
+static bool
 unify_trie(DECL_LD term_t t, trie *trie)
 { return PL_unify_atom(t, trie->symbol);
 }
 
-int
+bool
 get_trie(term_t t, trie **tp)
 { void *data;
   PL_blob_t *type;
@@ -1129,7 +1253,7 @@ get_trie(term_t t, trie **tp)
 }
 
 
-int
+bool
 get_trie_noex(term_t t, trie **tp)
 { void *data;
   PL_blob_t *type;
@@ -1145,7 +1269,7 @@ get_trie_noex(term_t t, trie **tp)
 }
 
 
-int
+bool
 trie_error(int rc, term_t culprit)
 { switch(rc)
   { case TRIE_LOOKUP_CONTAINS_ATTVAR:
@@ -1157,7 +1281,7 @@ trie_error(int rc, term_t culprit)
   }
 }
 
-int
+bool
 trie_trie_error(int rc, trie *trie)
 { GET_LD
   term_t t;
@@ -1175,7 +1299,7 @@ PRED_IMPL("trie_new", 1, trie_new, 0)
 
   if ( (trie = trie_create(NULL)) )
   { atom_t symbol = trie_symbol(trie);
-    int rc;
+    bool rc;
 
     rc = unify_trie(A1, trie);
     PL_unregister_atom(symbol);
@@ -1276,7 +1400,7 @@ unify_value(DECL_LD term_t t, word value)
 }
 
 
-int
+bool
 put_trie_value(DECL_LD term_t t, trie_node *node)
 { if ( !isRecord(node->value) )
   { *valTermRef(t) = node->value;
@@ -1287,7 +1411,7 @@ put_trie_value(DECL_LD term_t t, trie_node *node)
 }
 
 
-int
+bool
 set_trie_value_word(trie *trie, trie_node *node, word val)
 { if ( node->value )
   { if ( !equal_value(node->value, val) )
@@ -1314,7 +1438,7 @@ set_trie_value_word(trie *trie, trie_node *node, word val)
   }
 }
 
-int
+bool
 set_trie_value(DECL_LD trie *trie, trie_node *node, term_t value)
 { word val = intern_value(value);
 
@@ -1371,7 +1495,7 @@ trie_delete(trie *trie, trie_node *node, int prune)
  */
 
 #define trie_insert(Trie, Key, Value, nodep, update, abstract) LDFUNC(trie_insert, Trie, Key, Value, nodep, update, abstract)
-static int
+static bool
 trie_insert(DECL_LD term_t Trie, term_t Key, term_t Value, trie_node **nodep,
 	    int update, size_abstract *abstract)
 { trie *trie;
@@ -1389,7 +1513,7 @@ trie_insert(DECL_LD term_t Trie, term_t Key, term_t Value, trie_node **nodep,
     } else
     { if ( (Value  && isoff(trie, TRIE_ISMAP)) ||
 	   (!Value && isoff(trie, TRIE_ISSET)) )
-      { return PL_permission_error("insert", "trie", Trie);
+      { return PL_permission_error("insert", "trie", Trie),false;
       }
     }
 
@@ -1522,7 +1646,7 @@ PRED_IMPL("trie_update", 3, trie_update, 0)
 static
 PRED_IMPL("trie_insert", 4, trie_insert, 0)
 { PRED_LD
-  trie_node *node;
+  trie_node *node = NULL;	/* avoid warning */
 
   return ( trie_insert(A1, A2, A3, &node, false, NULL) &&
 	   PL_unify_pointer(A4, node) );
@@ -1611,61 +1735,79 @@ PRED_IMPL("trie_term", 2, trie_term, 0)
  */
 
 static void
-init_ukey_state(DECL_LD ukey_state *state, trie *trie, Word p)
+init_ukey_state(DECL_LD ukey_state *state, trie *trie, Word p, bool reinit)
 { state->trie = trie;
   state->ptr  = p;
   state->umode = uread;
-  state->max_var_seen = 0;
   state->a_offset = aTop-aBase;
+  if ( unlikely(reinit) )
+   emptyBuffer(&state->vars, (size_t)-1);
+  else
+    initBuffer(&state->vars);
 }
 
 static void
 destroy_ukey_state(DECL_LD ukey_state *state)
-{ if ( state->max_var_seen && state->vars != state->var_buf )
-    PL_free(state->vars);
+{ discardBuffer(&state->vars);
   aTop = aBase + state->a_offset;
 }
 
-static Word*
-find_var(ukey_state *state, size_t index)
-{ if ( index > state->max_var_seen )
-  { assert(index == state->max_var_seen+1);
+typedef struct varinfo
+{ Word address;			/* Address of the var/value */
+#if O_TRIE_ATTVAR
+  Word attributes;		/* Location of attributes */
+#endif
+} varinfo;
 
-    if ( !state->max_var_seen )
-    { state->vars_allocated = NVARS_FAST;
-      state->vars = state->var_buf;
-    } else if ( index >= state->vars_allocated )
-    { if ( state->vars == state->var_buf )
-      { state->vars = PL_malloc(sizeof(*state->vars)*NVARS_FAST*2);
-	memcpy(state->vars, state->var_buf, sizeof(*state->vars)*NVARS_FAST);
-      } else
-      { state->vars = PL_realloc(state->vars,
-				 sizeof(*state->vars)*state->vars_allocated*2);
-      }
-      state->vars_allocated *= 2;
-    }
-    state->vars[index] = NULL;
-    state->max_var_seen = index;
+static varinfo*
+find_var(ukey_state *state, size_t index) /* index is 1.. */
+{ if ( index > entriesBuffer(&state->vars, varinfo) )
+  { varinfo *vi = allocFromBuffer(&state->vars, sizeof(varinfo));
+    memset(vi, 0, sizeof(*vi));
   }
 
-  return &state->vars[index];
+  return &baseBuffer(&state->vars, varinfo)[index-1];
 }
 
+/**
+ * Take one  step in unifying  a term with  a sequence of  trie nodes.
+ * `state->ptr` is where  we are in the  term and `key` is  the key of
+ * the next  trie node to  consider.  The  `state` holds the  state of
+ * this state machine.
+ *
+ * Like the VM  unification, this engine uses an  _argument stack_ and
+ * distinguishes _read_  and _write_ mode.   We are in _read_  mode as
+ * long as  we are  matching an existing  structure.  If  the existing
+ * structure is  a variable and the  key is a functor,  we process the
+ * arguments in _write_ mode.
+ *
+ * Handling attributed variables is complicated.  In _write_ mode, the
+ * first encounter creates the attvar and subsequent create references
+ * to it.   In _read_ mode,  we cannot simply unify  as we do  not yet
+ * have  the attributes.   Thus,  we push  the  term-location and  the
+ * attvar to `state->attvars`  and at the end we deal  with it.  If we
+ * find  an attvar  in read  mode for  the second  time, we  unify the
+ * corresponding term values.
+ */
 
-static int
+static int			/* bool or *_OVERFLOW */
 unify_key(DECL_LD ukey_state *state, word key)
 { Word p = state->ptr;
 
+#define POPN(n) do {					    \
+    Word __wp;						    \
+    aTop -= n;						    \
+    __wp = *aTop;					    \
+    state->umode = ((int)(uintptr_t)__wp & uwrite);	    \
+    state->ptr   = (Word)((intptr_t)__wp &~uwrite);	    \
+  } while(0)
+
   switch(tagex(key))
   { case TAG_VAR|STG_LOCAL:			/* RESERVED_TRIE_VAL */
     { size_t popn = IS_TRIE_KEY_POP(key);
-      Word wp;
 
       assert(popn);
-      aTop -= popn;
-      wp = *aTop;
-      state->umode = ((int)(uintptr_t)wp & uwrite);
-      state->ptr   = (Word)((intptr_t)wp&~uwrite);
+      POPN(popn);
 
       DEBUG(MSG_TRIE_PUT_TERM,
 	    Sdprintf("U Popped(%zd) %zd, mode=%d\n",
@@ -1745,32 +1887,32 @@ unify_key(DECL_LD ukey_state *state, word key)
     assert(0);
     case TAG_VAR:
     { size_t index = (size_t)(key>>LMASK_BITS);
-      Word *v = find_var(state, index);
+      varinfo *vi = find_var(state, index);
 
       DEBUG(MSG_TRIE_PUT_TERM,
 	    { char b1[64]; char b2[64];
 	      Sdprintf("var %zd at %s (v=%p, *v=%s)\n",
 		       index,
 		       print_addr(state->ptr,b1),
-		       v, print_addr(*v,b2));
+		       vi, print_addr(vi->address,b2));
 	    });
 
       if ( state->umode == uwrite )
-      { if ( !*v )
+      { if ( !vi->address )
 	{ setVar(*state->ptr);
-	  *v = state->ptr;
+	  vi->address = state->ptr;
 	} else
-	{ *state->ptr = makeRefG(*v);
+	{ *state->ptr = makeRefG(vi->address);
 	}
       } else
       { deRef(p);
 
-	if ( !*v )
-	{ *v = state->ptr;
+	if ( !vi->address )
+	{ vi->address = state->ptr;
 	} else
 	{ int rc;
 
-	  if ( (rc=unify_ptrs(state->ptr, *v, ALLOW_RETCODE)) != true )
+	  if ( (rc=unify_ptrs(state->ptr, vi->address, ALLOW_RETCODE)) != true )
 	    return rc;
 	}
       }
@@ -1778,6 +1920,70 @@ unify_key(DECL_LD ukey_state *state, word key)
       break;
     }
     assert(0);
+#if O_TRIE_ATTVAR
+    case TAG_ATTVAR|STG_STATIC:
+    { size_t index = (size_t)(key>>LMASK_BITS);
+      varinfo *vi = find_var(state, index);
+
+      DEBUG(MSG_TRIE_PUT_TERM,
+	    Sdprintf("U Attvar %zd at %p\n", index, vi->address));
+
+      if ( !vi->address )
+      { pushArgumentStack((Word)((intptr_t)(p + 1)|state->umode));
+	if ( hasGlobalSpace(0) )
+	{ Word gp = gTop;
+	  register_attvar(gp);
+	  gp[1] = consPtr(&gp[2], TAG_ATTVAR|STG_GLOBAL);
+	  state->ptr = &gp[2];
+	  gTop += 3;
+	  vi->address = p;
+	  vi->attributes = &gp[1];
+	  if ( state->umode == uwrite )
+	    *p = makeRefG(&gp[1]);
+	  else
+	    state->umode = uwrite; /* wait for end (TAG_ATTVAR|STG_RESERVED) */
+
+	  return true;
+	} else
+	  return GLOBAL_OVERFLOW;
+      } else
+      { if ( state->umode == uwrite )
+	{ *state->ptr = makeRefG(vi->attributes);
+	} else
+	{ int rc = unify_ptrs(vi->address, p, ALLOW_RETCODE);
+	  if ( rc != true )
+	    return rc;
+	}
+	break;
+      }
+    }
+    case TAG_ATTVAR|STG_RESERVED:
+    { if ( state->umode == uread )
+      { size_t index = (size_t)(key>>LMASK_BITS);
+	varinfo *vi = find_var(state, index);
+	if ( hasGlobalSpace(0) )
+	{ Word p2;
+
+	  deRef2(vi->address, p2);
+	  if ( !isVar(*p2) )
+	  { if ( ison(state->trie, TRIE_ISTABLE) )
+	    { TrailAssignment(p2);
+	      *p2 = makeRefG(vi->attributes);
+	    } else
+	    { assignAttVar(vi->attributes, p2);
+	    }
+	  } else
+	  { int rc = unify_ptrs(vi->attributes, vi->address, ALLOW_RETCODE);
+	    if ( rc != true )
+	      return rc;
+	  }
+	} else
+	  return GLOBAL_OVERFLOW;
+      }
+      return true;
+    }
+#endif /*O_TRIE_ATTVAR*/
+    assert(0);
     case STG_GLOBAL|TAG_INTEGER:		/* indirect data */
     case STG_GLOBAL|TAG_STRING:
     case STG_GLOBAL|TAG_FLOAT:
@@ -1811,7 +2017,10 @@ unify_key(DECL_LD ukey_state *state, word key)
       pushVolatileAtom(word2atom(key));
       /*FALLTHROUGH*/
     case TAG_INTEGER:
-    { if ( state->umode == uwrite )
+    { DEBUG(MSG_TRIE_PUT_TERM,
+	    char buf[64];
+	    Sdprintf("U %s\n", print_val(key, buf)));
+      if ( state->umode == uwrite )
       { *p = key;
       } else
       { deRef(p);
@@ -1835,6 +2044,8 @@ unify_key(DECL_LD ukey_state *state, word key)
   state->ptr++;
 
   return true;
+
+#undef POPN
 }
 
 
@@ -2010,10 +2221,17 @@ add_choice(DECL_LD trie_gen_state *state, descent_state *dstate, trie_node *node
 	if ( !has_key ||
 	     k == children.key->key ||
 	     tagex(children.key->key) == TAG_VAR ||
+#if O_TRIE_ATTVAR
+	     tagex(children.key->key) == (TAG_ATTVAR|STG_STATIC) ||
+#endif
 	     IS_TRIE_KEY_POP(children.key->key) )
 	{ word key = children.key->key;
 
-	  if ( tagex(children.key->key) == TAG_VAR )
+	  if ( tagex(children.key->key) == TAG_VAR
+#if O_TRIE_ATTVAR
+	       || tagex(children.key->key) == (TAG_ATTVAR|STG_STATIC)
+#endif
+	     )
 	    dstate->prune = false;
 
 	  ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
@@ -2197,18 +2415,19 @@ Unify term with the term represented a trie path (list of trie_choice).
 Returns one of true, false or *_OVERFLOW.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define unify_trie_path(term, tn, gstate) LDFUNC(unify_trie_path, term, tn, gstate)
-static int
+#define unify_trie_path(term, tn, gstate) \
+	LDFUNC(unify_trie_path, term, tn, gstate)
+
+static int		      /* bool or _*OVERFLOW */
 unify_trie_path(DECL_LD term_t term, trie_node **tn, trie_gen_state *gstate)
-{ ukey_state ustate;
+{ int rc;
+  ukey_state ustate;
   trie_choice *ch = base_choice(gstate);
   trie_choice *top = top_choice(gstate);
 
-  init_ukey_state(&ustate, gstate->trie, valTermRef(term));
+  init_ukey_state(&ustate, gstate->trie, valTermRef(term), false);
   for( ; ch < top; ch++ )
-  { int rc;
-
-    if ( (rc=unify_key(&ustate, ch->key)) != true )
+  { if ( (rc=unify_key(&ustate, ch->key)) != true )
     { destroy_ukey_state(&ustate);
       return rc;
     }
@@ -2216,6 +2435,7 @@ unify_trie_path(DECL_LD term_t term, trie_node **tn, trie_gen_state *gstate)
 
   destroy_ukey_state(&ustate);
   *tn = ch[-1].child;
+  DEBUG(0, PL_check_data(term));
 
   return true;
 }
@@ -2223,7 +2443,7 @@ unify_trie_path(DECL_LD term_t term, trie_node **tn, trie_gen_state *gstate)
 
 foreign_t
 trie_gen_raw(trie *trie, trie_node *root, term_t Key, term_t Value,
-	     term_t Data, int LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
+	     term_t Data, bool LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
 	     void *ctx, control_t PL__ctx)
 { PRED_LD
   trie_gen_state state_buf;
@@ -2341,7 +2561,7 @@ next:;
 
 foreign_t
 trie_gen(term_t Trie, term_t Root, term_t Key, term_t Value,
-	 term_t Data, int LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
+	 term_t Data, bool LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
 	 void *ctx, control_t PL__ctx)
 { if ( CTX_CNTRL == FRG_FIRST_CALL )
   { trie *trie;
@@ -2380,8 +2600,10 @@ PRED_IMPL("trie_gen", 2, trie_gen, PL_FA_NONDETERMINISTIC)
 { return trie_gen(A1, 0, A2, 0, 0, NULL, NULL, PL__ctx);
 }
 
-#define unify_node_id(t, answer, ctx) LDFUNC(unify_node_id, t, answer, ctx)
-static int
+#define unify_node_id(t, answer, ctx) \
+	LDFUNC(unify_node_id, t, answer, ctx)
+
+static bool
 unify_node_id(DECL_LD term_t t, trie_node *answer, void *ctx)
 { (void) ctx;
 
@@ -2400,7 +2622,7 @@ PRED_IMPL("$trie_property", 2, trie_property, 0)
 { PRED_LD
   trie *trie;
 
-#ifdef O_TRIE_STATS
+#if O_TRIE_STATS
   static atom_t ATOM_lookup_count = 0;
   static atom_t ATOM_gen_call_count = 0;
   static atom_t ATOM_invalidated = 0;
@@ -2453,7 +2675,7 @@ PRED_IMPL("$trie_property", 2, trie_property, 0)
       { trie_stats stats;
 	stat_trie(trie, &stats);
 	return PL_unify_int64(arg, stats.hashes);
-#ifdef O_TRIE_STATS
+#if O_TRIE_STATS
       } else if ( name == ATOM_lookup_count )
       { return PL_unify_int64(arg, trie->stats.lookups);
       } else if ( name == ATOM_gen_call_count)
@@ -2604,11 +2826,14 @@ PRED_IMPL("trie_lookup_delete", 3, trie_lookup_delete, 0)
 
 typedef struct trie_compile_state
 { trie	       *trie;				/* Trie we are working on */
-  int		try;				/* There are alternatives */
-  int		last_is_fail;			/* Ends in I_FAIL */
-  tmp_buffer	codes;				/* Output instructions */
+  bool		try;				/* There are alternatives */
+  bool		last_is_fail;			/* Ends in I_FAIL */
+#if O_TRIE_ATTVAR
+  bool		has_attvars;			/* Trie contains attvars */
+#endif
   size_t	else_loc;			/* last else */
   size_t	maxvar;				/* Highest var index */
+  tmp_buffer	codes;				/* Output instructions */
 } trie_compile_state;
 
 static void
@@ -2730,26 +2955,48 @@ add_vmi_else_i(trie_compile_state *state, vmi c1, vmi c2, word w)
 #endif
 }
 
+/* If the  trie is a  map (rather than  a set), the  non-compiled trie
+ * stores  the  value  as  a  record.   This  function  is  called  by
+ * compile_trie_node()  on the  value extracted  from the  record.  It
+ * translates the value into T_* instructions that are very similar to
+ * the H_* unification instructions.
+ */
 
 #define compile_trie_value(v, state) LDFUNC(compile_trie_value, v, state)
-static int
+
+static bool
 compile_trie_value(DECL_LD Word v, trie_compile_state *state)
 { term_agenda_P agenda;
   size_t var_number = 0;
+  size_t voffset = state->maxvar;
   tmp_buffer varb;
   int rc = true;
-  int compounds = 0;
+  ssize_t compounds = 0;
   Word p;
+#if O_TRIE_ATTVAR
+  tmp_buffer attvarb;
+  TmpBuffer attvars = NULL;
+  attvar_mark *avm = NULL;
+#endif
 
   initTermAgenda_P(&agenda, 1, v);
   while( (p=nextTermAgenda_P(&agenda)) )
   { size_t popn;
 
     if ( (popn = IS_AC_TERM_POP(p)) )
-    { if ( popn == 1 )
+    { compounds -= popn;
+
+      if ( popn == 1 )
 	add_vmi(state, T_POP);
       else
 	add_vmi_d(state, T_POPN, (code)popn);
+
+#if O_TRIE_ATTVAR
+      if ( avm && avm->compound_depth == compounds )
+      { size_t index = (*avm->attvar)>>LMASK_BITS;
+	add_vmi_d(state, T_ATTVARZ, (code)index);
+      }
+#endif
     } else
     { word w = *p;
 
@@ -2759,20 +3006,56 @@ compile_trie_value(DECL_LD Word v, trie_compile_state *state)
 
 	  if ( isVar(w) )
 	  { if ( var_number++ == 0 )
-	    { initBuffer(&varb);
-	    }
+	      initBuffer(&varb);
+	    index = var_number+voffset;
+	    if ( index > state->maxvar )
+	      state->maxvar = index;
 	    addBuffer(&varb, p, Word);
-	    *p = w = ((((word)var_number))<<LMASK_BITS)|TAG_VAR;
+	    *p = w = ((((word)index))<<LMASK_BITS)|TAG_VAR;
+	  } else
+	  { index = (size_t)(w>>LMASK_BITS);
 	  }
-	  index = (size_t)(w>>LMASK_BITS);
-	  if ( index > state->maxvar )
-	    state->maxvar = index;
 	  add_vmi_d(state, T_VAR, (code)index);
 	  break;
 	}
 	case TAG_ATTVAR:
+#if O_TRIE_ATTVAR
+	  if ( tagex(w) != (TAG_ATTVAR|STG_STATIC) )
+	  { Word ap = valPAttVar(w);
+	    if ( var_number++ == 0 )
+	      initBuffer(&varb);
+	    size_t index = var_number+voffset;
+	    if ( index > state->maxvar )
+	      state->maxvar = var_number;
+	    state->has_attvars = true;
+
+	    addBuffer(&varb, p, Word);
+	    addBuffer(&varb, ap, Word);
+	    w = ((((word)index))<<LMASK_BITS)|TAG_ATTVAR|STG_STATIC;
+	    *p = w;
+	    add_vmi_d(state, T_ATTVARA, (code)index);
+	    if ( !attvars )
+	    { attvars = &attvarb;
+	      initBuffer(attvars);
+	    }
+	    avm = allocFromBuffer(attvars, sizeof(*avm));
+	    avm->compound_depth = compounds;
+	    avm->attvar = p;
+	    DEBUG(MSG_TRIE_PUT_TERM,
+		  Sdprintf("Opened attvar at %p\n", p));
+	    compounds++;
+	    pushWorkAgenda_P0(&agenda, 1, ap);
+	  } else
+	  { DEBUG(MSG_TRIE_PUT_TERM,
+		  Sdprintf("Shared attvar at %p\n", p));
+	    size_t index = w>>LMASK_BITS;
+	    add_vmi_d(state, T_ATTVARA, (code)index);
+	  }
+	  break;
+#else
 	  rc = TRIE_LOOKUP_CONTAINS_ATTVAR;
 	  goto out;
+#endif
 	case TAG_ATOM:			/* TBD: register */
 	  add_vmi_d(state, T_ATOM, (code)w);
 	  break;
@@ -2822,9 +3105,25 @@ out:
 
     for(; pp < ep; pp++)
     { Word vp = *pp;
-      setVar(*vp);
+      if ( tag(*vp) == TAG_VAR )
+      { setVar(*vp);
+#if O_TRIE_ATTVAR
+      } else if ( tagex(*vp) == (TAG_ATTVAR|STG_STATIC) )
+      { Word ap = *++pp;
+	*vp = consPtr(ap, TAG_ATTVAR|STG_GLOBAL);
+#endif
+      }
     }
     discardBuffer(&varb);
+#if O_TRIE_ATTVAR
+    if ( attvars == &attvarb )
+      discardBuffer(attvars);
+#endif
+  }
+
+  if ( rc != true )
+  { rc = trie_error(rc, pushWordAsTermRef(v));
+    popTermRef();
   }
 
   return rc;
@@ -2832,11 +3131,12 @@ out:
 
 
 #define compile_trie_node(n, state) LDFUNC(compile_trie_node, n, state)
-static int
+
+static bool
 compile_trie_node(DECL_LD trie_node *n, trie_compile_state *state)
 { trie_children children;
   word key;
-  int rc;
+  bool rc;
 
 next:
   children = n->children;
@@ -2862,16 +3162,46 @@ next:
 	add_vmi_d(state, T_FUNCTOR, (code)key);
       break;
     }
+#if O_TRIE_ATTVAR
+    case TAG_ATTVAR|STG_STATIC:
+    case TAG_ATTVAR|STG_RESERVED:
+      state->has_attvars = true;
+#endif
+    /*FALLTHROUGH*/
     case TAG_VAR:
     { size_t index = (size_t)(key>>LMASK_BITS);
+      vmi c;
 
       if ( index > state->maxvar )
 	state->maxvar = index;
 
       if ( state->try )
-	add_vmi_else_d(state, T_TRY_VAR, (code)index);
-      else
-	add_vmi_d(state, T_VAR, (code)index);
+      { switch(tagex(key))
+	{ case TAG_VAR:                 c = T_TRY_VAR; break;
+	  case TAG_ATTVAR|STG_STATIC:   c = T_TRY_ATTVARA; break;
+	  case TAG_ATTVAR|STG_RESERVED:
+	    if ( ison(state->trie, TRIE_ISTABLE) )
+	      c = T_TRY_ATTVARZT;
+	    else
+	      c = T_TRY_ATTVARZ;
+	    break;
+	  default: assert(0);           c = 0;
+	}
+	add_vmi_else_d(state, c, (code)index);
+      } else
+      { switch(tagex(key))
+	{ case TAG_VAR:                 c = T_VAR; break;
+	  case TAG_ATTVAR|STG_STATIC:   c = T_ATTVARA; break;
+	  case TAG_ATTVAR|STG_RESERVED:
+	    if ( ison(state->trie, TRIE_ISTABLE) )
+	      c = T_ATTVARZT;
+	    else
+	      c = T_ATTVARZ;
+	    break;
+	  default: assert(0);		c = 0;
+	}
+	add_vmi_d(state, c, (code)index);
+      }
       break;
     }
     case STG_GLOBAL|TAG_INTEGER:		/* indirect data */
@@ -2955,9 +3285,9 @@ children:
 	    goto next;
 	  }
 
-	  if ( (rc=compile_trie_node(n, state)) != true )
+	  if ( !compile_trie_node(n, state) )
 	  { freeTableEnum(e);
-	    return rc;
+	    return false;
 	  }
 	  fixup_else(state);
 	}
@@ -2992,6 +3322,7 @@ children:
 	  PL_reset_term_refs(t2);
 	}
       }
+      add_vmi(state, T_CHECKWAKEUP);
       add_vmi(state, I_EXIT);
       state->last_is_fail = false;
     } else
@@ -3000,11 +3331,12 @@ children:
     }
   }
 
+  DEBUG(CHK_SECURE, checkStacks(NULL));
   return true;
 }
 
 
-static int
+static bool
 fixup_last_fail(trie_compile_state *state)
 { if ( state->last_is_fail )
     add_vmi(state, I_EXIT);	/* make sure the clause ends with I_EXIT */
@@ -3012,21 +3344,22 @@ fixup_last_fail(trie_compile_state *state)
 }
 
 
-static int
+static bool
 create_trie_clause(Definition def, Clause *cp, trie_compile_state *state)
 { size_t code_size = entriesBuffer(&state->codes, code);
   size_t size      = sizeofClause(code_size);
-  //size_t clsize    = size + SIZEOF_CREF_CLAUSE;
+//size_t clsize    = size + SIZEOF_CREF_CLAUSE;
   Clause cl;
 
   cl = PL_malloc_atomic(size);
   memset(cl, 0, sizeof(*cl));
   cl->predicate = def;
   cl->code_size = code_size;
-  cl->prolog_vars = TRIE_VAR_OFFSET + state->maxvar;
+  cl->prolog_vars = (clsize_t)(TRIE_VAR_OFFSET + state->maxvar);
   cl->variables = cl->prolog_vars;	/* 2: pseudo arity */
   set(cl, UNIT_CLAUSE);			/* no body */
-  memcpy(cl->codes, baseBuffer(&state->codes, code), sizeOfBuffer(&state->codes));
+  memcpy(cl->codes, baseBuffer(&state->codes, code),
+	 sizeOfBuffer(&state->codes));
   *cp = cl;
 
   ATOMIC_ADD(&GD->statistics.codes, cl->code_size);
@@ -3068,6 +3401,8 @@ retry:
 	      goto retry;
 	    }
 	  }
+	} else
+	{ dbref = ATOM_error;
 	}
 	PL_close_foreign_frame(fid);
       } else
