@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2024, University of Amsterdam
+    Copyright (c)  1985-2025, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -52,6 +52,7 @@
 #include "os/pl-utf8.h"
 #include "os/pl-prologflag.h"
 #include "os/pl-fmt.h"
+#include "pl-termset.h"
 #include <stdio.h>			/* sprintf() */
 #include <errno.h>
 #ifdef HAVE_LOCALE_H
@@ -64,14 +65,12 @@
 #include <ieeefp.h>
 #endif
 
-#ifdef fpclassify
-#define HAVE_FPCLASSIFY 1
-#endif
-
 typedef struct
 { unsigned int flags;			/* PL_WRT_* flags */
   int   max_depth;			/* depth limit */
   int   depth;				/* current depth */
+  int	max_text;			/* Limit length for atoms/strings */
+  bool  truncated;			/* max_depth was exceeded */
   atom_t integer_format;		/* How to format integers */
   atom_t float_format;			/* How to format floats */
   atom_t spacing;			/* Where to insert spaces */
@@ -85,7 +84,8 @@ typedef struct
 #define WRITE_OPTIONS_DEFAULTS \
 	{ .spacing = ATOM_standard, \
 	  .integer_format = ATOM_int_format_specifier, \
-	  .float_format = ATOM_float_format_specifier \
+	  .float_format = ATOM_float_format_specifier, \
+	  .max_text = -1 \
 	}
 
 #define W_OP_ARG	1		/* writeTerm() location argument */
@@ -148,12 +148,12 @@ varName(term_t t, char *name)
 }
 
 
-static int
+static bool
 atomIsVarName(atom_t a)
 { Atom atom = atomValue(a);
 
   if ( isoff(atom->type, PL_BLOB_TEXT) || atom->length == 0 )
-    fail;
+    return false;
   if ( isUCSAtom(atom) )
   { pl_wchar_t *w = (pl_wchar_t*)atom->name;
     size_t len = atom->length / sizeof(pl_wchar_t);
@@ -213,7 +213,7 @@ writeNumberVar(DECL_LD term_t t, write_options *options)
     char buf[32];			/* Max is H354745078340568300 */
 
     if ( n < 0 )
-    { sprintf(buf, "S_%" PRId64, -n);
+    { snprintf(buf, sizeof buf, "S_%" PRId64, -n);
     } else
     { int i = (int)(n % 26);
       int64_t j = n / 26;
@@ -222,7 +222,7 @@ writeNumberVar(DECL_LD term_t t, write_options *options)
       { buf[0] = (char)(i+'A');
 	buf[1] = EOS;
       } else
-      { sprintf(buf, "%c%" PRId64, i+'A', j);
+      { snprintf(buf, sizeof buf, "%c%" PRId64, i+'A', j);
       }
     }
 
@@ -427,7 +427,6 @@ PutString(const char *str, IOSTREAM *s)
   return true;
 }
 
-
 static bool
 PutComma(write_options *options)
 { if ( options->spacing == ATOM_next_argument )
@@ -459,6 +458,22 @@ PutStringN(const char *str, size_t length, IOSTREAM *s)
   return true;
 }
 
+/* Write `length` wide characters from `str`.  Note that
+ * `length` is in characters, __not__ wchar_t units.
+ */
+
+static bool
+PutWStringN(const wchar_t *str, size_t length, IOSTREAM *s)
+{ while(length-- > 0)
+  { int c;
+    str = get_wchar(str, &c);
+    if ( !Putc(c, s) )
+      return false;
+  }
+
+  return true;
+}
+
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PutOpenToken() inserts a space in the  output stream if the last-written
@@ -546,6 +561,23 @@ PutTokenN(const char *s, size_t len, IOSTREAM *stream)
   return true;
 }
 
+static int
+PutElipsis(IOSTREAM *s, bool first)
+{ int rc = true;
+
+  if ( Scanrepresent(0x2026, s) == 0 )
+  { if ( first && !(rc=PutOpenToken(0x2026, s)) )
+      return false;
+    if ( Sputcode(0x2026, s) == -1 )
+      return false;
+    return rc;
+  } else
+  { if ( first && !(rc=PutOpenToken('.', s)) )
+      return false;
+    TRY(PutString("...", s));
+    return rc;
+  }
+}
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PutOpenBrace()/PutCloseBrace() are used to put  additional braces around
@@ -638,24 +670,32 @@ putQuoted(int c, int quote, int flags, IOSTREAM *stream)
   return true;
 }
 
-
-
+#if O_ATTVAR
 static bool
-writeQuoted(IOSTREAM *stream, const char *text, size_t len, int quote,
-	    write_options *options)
-{ const unsigned char *s = (const unsigned char *)text;
+writeAttributes(term_t t, int prec, write_options *options, int flags)
+{ term_t name  = PL_new_term_ref();
+  term_t value = PL_new_term_ref();
+  term_t att   = PL_new_term_ref();
 
-  TRY(Putc(quote, stream));
+  for(bool first=true; PL_is_functor(t, FUNCTOR_att3); first = false)
+  { _PL_get_arg(1, t, name);
+    _PL_get_arg(2, t, value);
+    _PL_get_arg(3, t, t);
 
-  while(len-- > 0)
-  { TRY(putQuoted(*s++, quote, options->flags, stream));
+    if ( !first && !PutComma(options) )
+      return false;
+    if ( !PL_cons_functor(att, FUNCTOR_colon2, name, value) ||
+	 !writeTerm(att, 999, options, W_COMPOUND_ARG) )
+      return false;
   }
+  if ( !PL_get_nil(t) &&
+       !writeTerm(t, 999, options, W_COMPOUND_ARG) )
+    return false;
 
-  return Putc(quote, stream);
+  PL_reset_term_refs(name);
+  return true;
 }
 
-
-#if O_ATTVAR
 static bool
 writeAttVar(term_t av, write_options *options)
 { GET_LD
@@ -664,7 +704,9 @@ writeAttVar(term_t av, write_options *options)
   TRY(PutToken(varName(av, buf), options->out));
 
   if ( (options->flags & PL_WRT_ATTVAR_DOTS) )
-  { return PutString("{...}", options->out);
+  { return (Putc('{', options->out) &&
+	    PutElipsis(options->out, false) &&
+	    Putc('}', options->out));
   } else if ( (options->flags & PL_WRT_ATTVAR_WRITE) )
   { fid_t fid;
     term_t a;
@@ -675,14 +717,14 @@ writeAttVar(term_t av, write_options *options)
     Sputcode('{', options->out);
     a = PL_new_term_ref();
     PL_get_attr(av, a);
-    if ( !writeTerm(a, 1200, options, W_TOP) )
+    if ( !writeAttributes(a, 1200, options, W_TOP) )
       return false;
     Sputcode('}', options->out);
     PL_close_foreign_frame(fid);
 
     return true;
   } else if ( (options->flags & PL_WRT_ATTVAR_PORTRAY) &&
-	      GD->cleaning <= CLN_PROLOG )
+	      GD->halt.cleaning <= CLN_PROLOG )
   { predicate_t pred;
     IOSTREAM *old;
     wakeup_state wstate;
@@ -729,6 +771,143 @@ writeBlob(atom_t a, write_options *options)
   return PutString(">", options->out);
 }
 
+static const wchar_t *
+wcs_backskip(const wchar_t *s, size_t len)
+{
+#if SIZEOF_WCHAR_T == 2
+  while(len-- > 0)
+  { s--;
+    if ( IS_UTF16_TRAIL(s[0]) )
+      s--;
+  }
+  return s;
+#else
+  return s-len;
+#endif
+}
+
+static bool
+write_chars(const char *str, size_t len, int quote, write_options *options)
+{ if ( quote )
+  { const unsigned char *s = (const unsigned char*)str;
+    const unsigned char *e = &s[len];
+
+    while(s<e)
+    { int chr = *s++;
+
+      TRY(putQuoted(chr, quote, options->flags, options->out));
+    }
+
+    return true;
+  } else
+  { return PutStringN(str, len, options->out);
+  }
+}
+
+static bool
+write_wchars(const wchar_t *s, size_t len, int quote, write_options *options)
+{ if ( quote )
+  { while(len-- > 0)
+    { int chr;
+
+      s = get_wchar(s, &chr);
+
+      TRY(putQuoted(chr, quote, options->flags, options->out));
+    }
+    return true;
+  } else
+  { return PutWStringN(s, len, options->out);
+  }
+}
+
+
+static int					/* false, true, TRUE_WITH_SPACE */
+writeText(PL_chars_t *txt, int quote, write_options *options)
+{ int rc = true;
+  size_t len = PL_text_length(txt);
+
+  if ( len == 0 )
+    return true;
+
+  if ( options->max_text >= 0 )
+  { if ( len > options->max_text )
+    { int mt = options->max_text;
+      int sl = min(mt, mt/2);		/* suffix len */
+      int pl = mt-sl;			/* prefix len */
+      bool quoted_ellipsis = false;
+
+      options->truncated = true;
+
+      if ( !quote && ison(options, PL_WRT_QUOTED) )
+      { quoted_ellipsis = true;
+	quote = '\'';
+	TRY( (rc=PutOpenToken(quote, options->out)) &&
+	     Putc(quote, options->out) );
+      }
+
+      switch(txt->encoding)
+      { case ENC_ISO_LATIN_1:
+	{ const char *s = (const char*)txt->text.t;
+	  if ( pl > 0 )
+	  { if ( !quote )
+	      TRY(rc=PutOpenToken(s[0], options->out));
+	    TRY(write_chars(s, pl, quote, options));
+	    TRY(PutElipsis(options->out, false));
+	  } else
+	  { TRY(rc=PutElipsis(options->out, true));
+	  }
+	  TRY(write_chars(s+len-sl, sl, quote, options));
+	  break;
+	}
+	case ENC_WCHAR:
+	{ const wchar_t *s = txt->text.w;
+	  const wchar_t *e = &s[txt->length];
+	  const wchar_t *suffix = wcs_backskip(e, sl);
+
+	  if ( pl > 0 )
+	  { int c;
+	    get_wchar(s, &c);
+	    if ( !quote )
+	      TRY(rc = PutOpenToken(c, options->out));
+	    TRY(write_wchars(s, pl, quote, options));
+	    TRY(PutElipsis(options->out, false));
+	  } else
+	  { TRY(rc=PutElipsis(options->out, true));
+	  }
+	  TRY(write_wchars(suffix, sl, quote, options));
+	  break;
+	}
+        default:
+	  assert(0);
+	  return false;
+      }
+      if ( rc && quoted_ellipsis )
+	TRY(Putc(quote, options->out));
+      return rc;
+    }
+  }
+
+  switch(txt->encoding)
+  { case ENC_ISO_LATIN_1:
+    { const char *s = (const char*)txt->text.t;
+      if ( !quote )
+	TRY(rc=PutOpenToken(s[0], options->out));
+      TRY(write_chars(s, len, quote, options));
+      return rc;
+    }
+    case ENC_WCHAR:
+    { const wchar_t *s = (const wchar_t*)txt->text.t;
+      if ( !quote )
+	TRY(rc=PutOpenToken(s[0], options->out));
+      TRY(write_wchars(s, len, quote, options));
+      return rc;
+    }
+    default:
+      assert(0);
+      return false;
+  }
+}
+
 
 static int				/* false, true or TRUE_WITH_SPACE */
 writeAtom(atom_t a, write_options *options)
@@ -736,7 +915,7 @@ writeAtom(atom_t a, write_options *options)
 
   if ( (options->flags & PL_WRT_BLOB_PORTRAY) &&
        isoff(atom->type, PL_BLOB_TEXT) &&
-       GD->cleaning <= CLN_PROLOG &&
+       GD->halt.cleaning <= CLN_PROLOG &&
        a != ATOM_nil )
   { GET_LD
     int rc;
@@ -759,34 +938,47 @@ writeAtom(atom_t a, write_options *options)
     }
   }
 
+  if ( atom->type->write_ex )
+    return ((*atom->type->write_ex)(a, options) &&
+	    !Sferror(options->out));
   if ( atom->type->write )
     return ((*atom->type->write)(options->out, a, options->flags) &&
 	    !Sferror(options->out));
   if ( isoff(atom->type, PL_BLOB_TEXT) )
     return writeBlob(a, options);
 
+  PL_chars_t text;
+
+  if ( !get_atom_text(a, &text) )
+  { assert(0);
+    return false;
+  }
+
   if ( ison(options, PL_WRT_QUOTED) )
   { switch( atomType(a, options) )
     { case AT_LOWER:
       case AT_SYMBOL:
       case AT_SOLO:
       case AT_SPECIAL:
-	return PutToken(atom->name, options->out);
+	return writeText(&text, 0, options);
       case AT_QUOTE:
       case AT_FULLSTOP:
       default:
       { int rc;
+	int quote = '\'';
+
+	if ( !( (rc=PutOpenToken(quote, options->out)) &&
+		Putc(quote, options->out) &&
+		writeText(&text, quote, options) &&
+		Putc(quote, options->out) ) )
+	  return false;
 
-	TRY(rc=PutOpenToken('\'', options->out));
-	TRY(writeQuoted(options->out,
-			atom->name,
-			atom->length,
-			'\'', options));
 	return rc;
       }
     }
   } else
-    return PutTokenN(atom->name, atom->length, options->out);
+  { return writeText(&text, 0, options);
+  }
 }
 
 
@@ -800,43 +992,33 @@ writeAtomToStream(IOSTREAM *s, atom_t atom)
   return !!writeAtom(atom, &options);
 }
 
-
 bool
-writeUCSAtom(IOSTREAM *fd, atom_t atom, int flags)
-{ Atom a = atomValue(atom);
-  const pl_wchar_t *s = (const pl_wchar_t*)a->name;
-  size_t len = a->length/sizeof(pl_wchar_t);
-  const pl_wchar_t *e = &s[len];
+writeUCSAtom(atom_t atom, void *context)
+{ write_options *options = context;
+  PL_chars_t text;
 
-  if ( (flags&PL_WRT_QUOTED) && !unquoted_atomW(atom, fd, flags) )
-  { pl_wchar_t quote = L'\'';
-
-    TRY(PutOpenToken(quote, fd) &&
-	Putc(quote, fd));
-
-    while(s < e)
-    { int c;
-
-      s = get_wchar(s, &c);
-      TRY(putQuoted(c, quote, flags, fd));
-    }
-
-    return Putc(quote, fd);
+  if ( !get_atom_text(atom, &text) )
+  { assert(0);
+    return false;
   }
 
-  if ( e > s )
-  { int c;
+  if ( (options->flags&PL_WRT_QUOTED) &&
+       !unquoted_atomW(atom, options->out, options->flags) )
+  { pl_wchar_t quote = L'\'';
 
-    get_wchar(s, &c);
-    if ( !PutOpenToken(c, fd) )
+    return ( PutOpenToken(quote, options->out) &&
+	     Putc(quote, options->out) &&
+	     writeText(&text, quote, options) &&
+	     Putc(quote, options->out) );
+  } else
+  { PL_chars_t text;
+
+    if ( !get_atom_text(atom, &text) )
+    { assert(0);
       return false;
-
-    while ( s < e )
-    { s = get_wchar(s, &c);
-
-      if ( !Putc(c, fd) )
-	return false;
     }
+
+    return writeText(&text, 0, options);
   }
 
   return true;
@@ -901,72 +1083,15 @@ writeString(term_t t, write_options *options)
 
     if ( !(rc=Putc(quote, options->out)) )
       goto out;
-
-    switch(txt.encoding)
-    { case ENC_ISO_LATIN_1:
-      { const unsigned char *s = (const unsigned char*)txt.text.t;
-	const unsigned char *e = &s[txt.length];
-
-	while(s<e)
-	{ int chr = *s++;
-
-	  if ( !(rc=putQuoted(chr, quote, options->flags, options->out)) )
-	    goto out;
-	}
-	break;
-      }
-      case ENC_WCHAR:
-      { const wchar_t *s = txt.text.w;
-	const wchar_t *e = &s[txt.length];
-
-	while(s<e)
-	{ int chr;
-
-	  s = get_wchar(s, &chr);
-	  if ( !(rc=putQuoted(chr, quote, options->flags, options->out)) )
-	    goto out;
-	}
-	break;
-      }
-      default:
-	assert(0);
-    }
-    rc = Putc(quote, options->out);
+    rc = ( writeText(&txt, quote, options) &&
+	   Putc(quote, options->out) );
   } else
-  { switch(txt.encoding)
-    { case ENC_ISO_LATIN_1:
-      { const unsigned char *s = (const unsigned char*)txt.text.t;
-	const unsigned char *e = &s[txt.length];
-
-	while(s<e)
-	{ int chr = *s++;
-
-	  if ( !(rc=Putc(chr, options->out)) )
-	    goto out;
-	}
-	break;
-      }
-      case ENC_WCHAR:
-      { const wchar_t *s = txt.text.w;
-	const wchar_t *e = &s[txt.length];
-
-	while(s<e)
-	{ int chr;
-
-	  s = get_wchar(s, &chr);
-	  if ( !(rc=Putc(chr, options->out)) )
-	    goto out;
-	}
-	break;
-      }
-      default:
-	assert(0);
-    }
+  { rc = writeText(&txt, 0, options);
   }
-  PL_STRINGS_RELEASE();
 
 out:
   PL_free_text(&txt);
+  PL_STRINGS_RELEASE();
 
   return rc;
 }
@@ -1088,55 +1213,12 @@ writeINF(char *buf, size_t len, double f)
 
 static size_t
 format_special_float(char *buf, size_t len, double f)
-{
-#ifdef HAVE_FPCLASSIFY
-  switch(fpclassify(f))
+{ switch(fpclassify(f))
   { case FP_NAN:
       return writeNaN(buf, len, f);
     case FP_INFINITE:
       return writeINF(buf, len, f);
   }
-#else
-#ifdef HAVE_FPCLASS
-  switch(fpclass(f))
-  { case FP_SNAN:
-    case FP_QNAN:
-      return writeNaN(buf, len, f);
-    case FP_NINF:
-    case FP_PINF:
-      return writeINF(buf, len, f);
-    case FP_NDENORM:		/* pos/neg denormalized non-zero */
-    case FP_PDENORM:
-    case FP_NNORM:			/* pos/neg normalized non-zero */
-    case FP_PNORM:
-    case FP_NZERO:			/* pos/neg zero */
-    case FP_PZERO:
-      break;
-  }
-#else
-#ifdef HAVE__FPCLASS
-  switch(_fpclass(f))
-  { case _FPCLASS_SNAN:
-    case _FPCLASS_QNAN:
-      return writeNaN(buf, len, f);
-    case _FPCLASS_NINF:
-    case _FPCLASS_PINF:
-      return writeINF(buf, len, f);
-  }
-#else
-#ifdef HAVE_ISINF
-  if ( isinf(f) )
-  { return writeINF(buf, len, f);
-  } else
-#endif
-#ifdef HAVE_ISNAN
-  if ( isnan(f) )
-  { return writeNaN(buf, len, f);
-  }
-#endif
-#endif /*HAVE__FPCLASS*/
-#endif /*HAVE_FPCLASS*/
-#endif /*HAVE_FPCLASSIFY*/
 
   return 0;
 }
@@ -1478,7 +1560,7 @@ static int
 callPortray(term_t arg, int prec, write_options *options)
 { predicate_t pred;
 
-  if ( GD->cleaning > CLN_PROLOG )
+  if ( GD->halt.cleaning > CLN_PROLOG )
     fail;				/* avoid dangerous callbacks */
 
   if ( options->portray_goal )
@@ -1528,7 +1610,7 @@ callPortray(term_t arg, int prec, write_options *options)
 static bool
 writeTerm(term_t t, int prec, write_options *options, int flags)
 { GET_LD
-  int rval;
+  bool rval;
   int levelSave = options->depth;
   fid_t fid;
 
@@ -1541,8 +1623,9 @@ writeTerm(term_t t, int prec, write_options *options, int flags)
   }
 
   if ( ++options->depth > options->max_depth && options->max_depth )
-  { PutOpenToken('.', options->out);
-    rval = PutString("...", options->out);
+  { options->truncated = true;
+    PutOpenToken('.', options->out);
+    rval = PutElipsis(options->out, true);
   } else
   { rval = writeTerm2(t, prec, options, flags);
   }
@@ -1570,7 +1653,11 @@ writeList(term_t list, write_options *options)
       if ( PL_get_nil(l) )
 	break;
       if ( ++options->depth >= options->max_depth && options->max_depth )
-	return PutString("|...]", options->out);
+      { options->truncated = true;
+	return ( Putc('|', options->out) &&
+		 PutElipsis(options->out, false) &&
+		 Putc(']', options->out) );
+      }
       if ( !PL_is_functor(l, FUNCTOR_dot2) )
       { TRY(Putc('|', options->out));
 	TRY(writeTerm(l, 999, options, W_LIST_TAIL));
@@ -1608,7 +1695,8 @@ writeList(term_t list, write_options *options)
       }
 
       if ( ++options->depth >= options->max_depth && options->max_depth )
-      { if ( !PutToken("...", options->out) )
+      { options->truncated = true;
+	if ( !PutElipsis(options->out, true) )
 	  return false;
 	while(depth-->0)
 	{ if ( !Putc(')', options->out) )
@@ -1742,7 +1830,9 @@ writeTerm2(term_t t, int prec, write_options *options, int flags)
       return false;
     }
 					/* operators */
-    if ( isoff(options, PL_WRT_IGNOREOPS) )
+    if ( isoff(options, PL_WRT_IGNOREOPS) ||
+	 ( functor == ATOM_comma && arity == 2 &&
+	   ison(options, PL_WRT_INFIX_COMMA)) )
     { term_t arg;
 
       if ( !(arg=PL_new_term_ref()) )
@@ -1833,6 +1923,12 @@ writeTerm2(term_t t, int prec, write_options *options, int flags)
 	    { TRY(PutBar(options));
 	    } else if ( functor == ATOM_fdot )
 	    { TRY(PutToken(".", out));
+	    } else if ( functor == ATOM_divide &&
+			ison(options, PL_WRT_RAT_NATURAL) &&
+			PL_is_integer(arg) &&
+			_PL_get_arg(1, t, arg) &&
+			PL_is_number(arg) )
+	    { TRY(PutString(" / ", out));
 	    } else
 	    { switch(writeAtom(functor, options))
 	      { case false:
@@ -1897,6 +1993,7 @@ reunify_acyclic_substitutions(term_t substitutions, term_t cycles,
   while(PL_get_list(s_tail, s_head, s_tail))
   { _PL_get_arg(1, s_head, var);
     _PL_get_arg(2, s_head, value);
+
     if ( PL_var_occurs_in(var, value) )
     { if ( (options->flags&PL_WRT_NUMBERVARS) )
       { if ( !PL_unify_term(var,
@@ -1908,9 +2005,11 @@ reunify_acyclic_substitutions(term_t substitutions, term_t cycles,
       if ( !PL_unify_list(c_tail, c_head, c_tail) ||
 	   !PL_unify(c_head, s_head) )
 	return false;
-    } else
+    } else if ( !PL_exception(0) )
     { if ( !PL_unify(var, value) )
 	return false;
+    } else
+    { return false;
     }
   }
 
@@ -1918,10 +2017,10 @@ reunify_acyclic_substitutions(term_t substitutions, term_t cycles,
 }
 
 
-static int
+static bool
 writeTopTerm(term_t term, int prec, write_options *options)
 { GET_LD
-  int rc;
+  bool rc;
   int wflags;
 
   if ( ison(options, PL_WRT_PARTIAL) && prec != 999 && prec != 1200 )
@@ -1951,7 +2050,7 @@ writeTopTerm(term_t term, int prec, write_options *options)
 	 !PL_factorize_term(term, template, substitutions) ||
 	 !reunify_acyclic_substitutions(substitutions, cycles, options) ||
 	 !PL_unify_term(at_term,
-			PL_FUNCTOR, FUNCTOR_xpceref2,
+			PL_FUNCTOR, FUNCTOR_at_sign2,
 			  PL_TERM, template,
 			  PL_TERM, cycles) )
       return false;
@@ -1968,7 +2067,7 @@ writeTopTerm(term_t term, int prec, write_options *options)
 
 
 #define bind_varnames(names) LDFUNC(bind_varnames, names)
-static int
+static bool
 bind_varnames(DECL_LD term_t names)
 { term_t tail, head, var, namet;
   int check_cycle_after = 1000;
@@ -2043,6 +2142,7 @@ static const PL_option_t write_term_options[] =
 { { ATOM_quoted,		    OPT_BOOL },
   { ATOM_quote_non_ascii,	    OPT_BOOL },
   { ATOM_ignore_ops,		    OPT_BOOL },
+  { ATOM_portable,		    OPT_BOOL },
   { ATOM_dotlists,		    OPT_BOOL },
   { ATOM_brace_terms,		    OPT_BOOL },
   { ATOM_numbervars,		    OPT_BOOL },
@@ -2052,10 +2152,12 @@ static const PL_option_t write_term_options[] =
   { ATOM_character_escapes,	    OPT_BOOL },
   { ATOM_character_escapes_unicode, OPT_BOOL },
   { ATOM_max_depth,		    OPT_INT  },
+  { ATOM_max_text,		    OPT_INT  },
+  { ATOM_truncated,		    OPT_TERM },
   { ATOM_module,		    OPT_ATOM },
   { ATOM_back_quotes,		    OPT_ATOM },
   { ATOM_attributes,		    OPT_ATOM },
-  { ATOM_priority,		    OPT_INT },
+  { ATOM_priority,		    OPT_INT  },
   { ATOM_partial,		    OPT_BOOL },
   { ATOM_spacing,		    OPT_ATOM },
   { ATOM_blobs,			    OPT_ATOM },
@@ -2074,8 +2176,9 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
 { GET_LD
   int quoted      = false;
   int ignore_ops  = false;
+  int portable    = false;
   int dotlists    = false;
-  int braceterms  = true;
+  int braceterms  = -1;
   int numbervars  = -1;			/* not set */
   int portray     = false;
   term_t gportray = 0;
@@ -2093,15 +2196,17 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   int fullstop    = false;
   int no_lists    = false;
   term_t varnames = 0;
+  term_t truncated = 0;
   IOSTREAM *s = NULL;
   write_options options = WRITE_OPTIONS_DEFAULTS;
   int rc;
 
   if ( !PL_scan_options(opts, 0, "write_option", write_term_options,
-			&quoted, &quote_non_ascii, &ignore_ops, &dotlists, &braceterms,
+			&quoted, &quote_non_ascii, &ignore_ops, &portable,
+			&dotlists, &braceterms,
 			&numbervars, &portray, &portray, &gportray,
 			&charescape, &charescape_unicode,
-			&options.max_depth, &mname,
+			&options.max_depth, &options.max_text, &truncated, &mname,
 			&bq, &attr, &priority, &partial, &options.spacing,
 			&blobs, &cycles, &varnames, &nl, &fullstop,
 			&no_lists,
@@ -2167,16 +2272,18 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( numbervars == -1 )
     numbervars = (portray ? true : false);
 
-  if ( quoted )          options.flags |= PL_WRT_QUOTED;
-  if ( quote_non_ascii ) options.flags |= PL_WRT_QUOTE_NON_ASCII;
-  if ( ignore_ops )      options.flags |= PL_WRT_IGNOREOPS;
-  if ( dotlists )        options.flags |= PL_WRT_DOTLISTS;
-  if ( !braceterms )     options.flags |= PL_WRT_BRACETERMS;
-  if ( numbervars )      options.flags |= PL_WRT_NUMBERVARS;
-  if ( portray )         options.flags |= PL_WRT_PORTRAY;
-  if ( !cycles )         options.flags |= PL_WRT_NO_CYCLES;
-  if ( no_lists )        options.flags |= PL_WRT_NO_LISTS;
-  if ( partial )	 options.flags |= PL_WRT_PARTIAL;
+  if ( quoted )              options.flags |= PL_WRT_QUOTED;
+  if ( quote_non_ascii )     options.flags |= PL_WRT_QUOTE_NON_ASCII;
+  if ( ignore_ops )          options.flags |= PL_WRT_IGNOREOPS|PL_WRT_BRACETERMS;
+  if ( portable )	     options.flags |= PL_WRT_PORTABLE;
+  if ( dotlists )            options.flags |= PL_WRT_DOTLISTS;
+  if ( braceterms == false ) options.flags |= PL_WRT_BRACETERMS;
+  if ( braceterms == true )  options.flags &= ~PL_WRT_BRACETERMS;
+  if ( numbervars )          options.flags |= PL_WRT_NUMBERVARS;
+  if ( portray )             options.flags |= PL_WRT_PORTRAY;
+  if ( !cycles )             options.flags |= PL_WRT_NO_CYCLES;
+  if ( no_lists )            options.flags |= PL_WRT_NO_LISTS;
+  if ( partial )	     options.flags |= PL_WRT_PARTIAL;
   if ( bq )
   { unsigned int flags = 0;
 
@@ -2216,6 +2323,9 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   else if ( nl )
     rc = Putc('\n', s);
 
+  if ( rc && truncated )
+    rc = PL_unify_bool_ex(truncated, options.truncated);
+
 out:
   END_NUMBERVARS(varnames);
 
@@ -2274,6 +2384,8 @@ do_write2(term_t stream, term_t term, int flags, int canonical)
       options.flags |= PL_WRT_CHARESCAPES;
     if ( ison(options.module, BQ_STRING) )
       options.flags |= PL_WRT_BACKQUOTED_STRING;
+    if ( ison(options.module, RAT_NATURAL) )
+      options.flags |= PL_WRT_RAT_NATURAL;
 
     PutOpenToken(EOF, s);		/* reset this */
     rc = writeTopTerm(term, 1200, &options);
@@ -2505,7 +2617,7 @@ PRED_IMPL("write_length", 3, write_length, 0)
 
   if ( (s = Snew(&lss, sflags, (IOFUNCTIONS *)&lss_functions)) )
   { int64_t len;
-    int rc;
+    size_t rc;
     pl_features_t oldmask = LD->prolog_flag.mask; /* (*) */
 
     lss.stream = s;
